<?php

class RegisteroneShortcodes
{
    public $base_request_url = '';
    public $base_site_url = '';
    public $plugin_base_name = 'dwc';
    public $plugin_api_auth_key;

    private $application_steps;
    private $current_screen;
    private $attributes;
    private $event_attributes;
    private $form_steps;
    private $backup_attributes;
    private $default_args = ['method'=>'', 'timeout'=>60, 'headers'=>'', 'body'=>null, 'compress'=>false, 'decompress'=>true, 'sslverify'=>false];
    private $default_headers = ["Content-Type"=>"application/json; charset=utf-8"];

    /**
     * RegisteroneShortcodes constructor.
     */
    public function __construct()
    {
        $account_information = get_option($this->plugin_base_name . '_account_information');
        $api_auth_key = (isset($account_information['privateAPIkey'])) ? $account_information['privateAPIkey'] . ':' : '';
        $this->footer_powered_by_option = (isset($account_information['displayPoweredBy'])) ? $account_information['displayPoweredBy'] : 0;

        if(isset($_SERVER['SERVER_ADDR'])) {
            $ip_addr = explode(".", $_SERVER['SERVER_ADDR']);
            if ($ip_addr[0] == '192' && $ip_addr[1] == '168') {
                $this->base_request_url = "http://api.registerone.net/v1/apply/";
                $this->base_site_url = "https://app.registerone.net";
            } else {
                $this->base_request_url = "https://api.registerone.com/v1/apply/";
                $this->base_site_url = "https://app.registerone.com";
            }
        }else{
            $this->base_request_url = "https://api.registerone.com/v1/apply/";
            $this->base_site_url = "https://app.registerone.com";
        }

        $this->plugin_api_auth_key = base64_encode($api_auth_key);

        add_action('wp_ajax_dwc_form_submission', array($this, 'handle_form_submission'));
        add_action('wp_ajax_nopriv_dwc_form_submission', array($this, 'handle_form_submission'));

        add_action('wp_ajax_dwc_map_submission', array($this, 'handle_map_submission'));
        add_action('wp_ajax_nopriv_dwc_map_submission', array($this, 'handle_map_submission'));

        add_action('wp_ajax_dwc_signature_submission', array($this, 'handle_signature_submission'));
        add_action('wp_ajax_nopriv_dwc_signature_submission', array($this, 'handle_signature_submission'));

        add_shortcode('registerone_form', array($this, 'registerone_form_shortcode'));

        if(isset($_SERVER['SERVER_NAME'])) {
            if (stristr($_SERVER['SERVER_NAME'],"forms.registerone")) {
                session_start();
                if(!empty($_SESSION['title'])) {
                    add_filter('pre_get_document_title', 'registerone_change_page_title');
                    function registerone_change_page_title()
                    {
                        $title = $_SESSION['title'] . " - RegisterONE";
                        unset($_SESSION['title']);
                        return $title;
                    }
                }
            }
        }
    }

    /**
     * @param $atts
     * @return string
     */
    public function registerone_form_shortcode($atts)
    {
        global $wp_query;
        $this->current_screen = $this->_get_current_screen();
        $request_url = $is_hosted_form = "";
        $register_url = false;

        $attributes = shortcode_atts(array(
            'form_key' => '',
            'map_key' => '',
        ), $atts, 'registerone_form');

        // Dynamic form_key with url slug
        $attributes['form_key'] = $attributes['form_key'] == 'dynamic' ? $wp_query->query_vars['form_key'] : $attributes['form_key'];

        // Dynamic form_key with url slug
        $attributes['id_hash'] = $wp_query->query_vars['id_hash'] ? $wp_query->query_vars['id_hash'] : 0;

        $headers = $this->_set_request_header(['Accept' => "application/json, text/javascript, */*; q=0.01", 'Authorization' => 'Basic ' . $this->plugin_api_auth_key]);

        $request_args = $this->_get_request_args([
                'method' => 'GET',
                'headers' => $headers
            ]
        );

        if(isset($attributes['map_key']) && $attributes['map_key'] != ''){

            $this->current_screen = 'mapview';

            $map_key = $attributes['map_key'];
            list($event_code, $amenity_id) = explode("-",$map_key,2);
            $request_url = $this->base_request_url . $this->current_screen . "/" . $event_code . "/" . $amenity_id;

            if (isset($_GET['kid']) && $_GET['kid'] != null) {
                $request_url .= "/" . $_GET['kid'];
            }

        }else{

            if (isset($_GET['pk']) && $_GET['pk'] != null) {
                if (isset($_GET['rfc']) && $_GET['rfc'] != null) {
                    $request_url_event_attributes = $this->base_request_url . $attributes['id_hash'] . "/" . $attributes['form_key'] . "/profile/" . $_GET['pk'] . "/" . $_GET['rfc'];
                }else{
                    $request_url_event_attributes = $this->base_request_url . $attributes['id_hash'] . "/" . $attributes['form_key'] . "/profile/" . $_GET['pk'];
                }
            } else {
                $request_url_event_attributes = $this->base_request_url . $attributes['id_hash'] . "/" . $attributes['form_key'] . "/profile";
            }

            $this->event_attributes = wp_remote_retrieve_body(wp_remote_request($request_url_event_attributes, $request_args));

            $attr_tmp = json_decode($this->event_attributes, true);

            if (isset($attr_tmp['event_a'])) {

                wp_register_style('dwc frontend body-bg css', REGISTERONE_WORDPRESS_CLIENT_URL . '/front-end/assets/css/body-bg.css');
                wp_enqueue_style('dwc frontend body-bg css');

                if ($attr_tmp['event_a']['bg_img_url'] != null) {
                    $custom_css = "
                    body {
                        background-image: url('{$attr_tmp['event_a']['bg_img_url']}');
                        background-repeat: repeat;
                    }";
                    wp_add_inline_style('dwc frontend body-bg css', $custom_css);
                }

                if (isset($_SERVER['SERVER_NAME'])) {
                    if (stristr($_SERVER['SERVER_NAME'], "forms.registerone")) {
                        $is_hosted_form = true;
                        if (!empty($attr_tmp['event_a']['title'])) {
                            session_start();
                            $_SESSION['title'] = $attr_tmp['event_a']['title'];
                            session_write_close();
                        }
                    }
                }
            }

            $this->backup_attributes['profile'] = $this->event_attributes;

            $request_url_model = $this->base_request_url . $attributes['id_hash'] . "/" . $attributes['form_key'] . "/" . $this->current_screen . "/attributes";
            $this->attributes[$this->current_screen] = wp_remote_retrieve_body(wp_remote_request($request_url_model, $request_args));

            if (isset($_GET['maprel']) && $_GET['pk'] != null) {

                if (isset($_GET['pk']) && $_GET['pk'] != null) {
                    $request_url = $this->base_request_url . $attributes['id_hash'] . "/" . $attributes['form_key'] . "/" . $this->current_screen . "/release/" . $_GET['maprel'] . "/" . $_GET['pk'];
                } else {
                    $request_url = $this->base_request_url . $attributes['id_hash'] . "/" . $attributes['form_key'] . "/" . $this->current_screen;
                }

            } elseif ($this->current_screen == "map") {

                if (isset($_GET['pk']) && $_GET['pk'] != null) {
                    $request_url = $this->base_request_url . $attributes['id_hash'] . "/" . $attributes['form_key'] . "/" . $this->current_screen . "/" . $_GET['pk'] . "/" . $_GET['map'];
                } else {
                    $request_url = $this->base_request_url . $attributes['id_hash'] . "/" . $attributes['form_key'] . "/" . $this->current_screen;
                }

            } else {

                if (isset($_GET['pk']) && $_GET['pk'] != null) {

                    if(isset($_GET['rfc']) && $_GET['rfc'] != null) {
                        $request_url = $this->base_request_url . $attributes['id_hash'] . "/" . $attributes['form_key'] . "/" . $this->current_screen . "/" . $_GET['pk'] . "/" . $_GET['rfc'];
                    }else{
                        $request_url = $this->base_request_url . $attributes['id_hash'] . "/" . $attributes['form_key'] . "/" . $this->current_screen . "/" . $_GET['pk'];
                    }

                } elseif (isset($_GET['return']) && $_GET['return'] != null) {
                    $request_url = $this->base_request_url . $attributes['id_hash'] . "/" . $attributes['form_key'] . "/return/1";

                } else {
                    $request_url = $this->base_request_url . $attributes['id_hash'] . "/" . $attributes['form_key'] . "/" . $this->current_screen;
                    $register_url = true;
                }
            }
        }

        if($register_url){
            if($form_url = $this->_get_base_url()) {
                $request_args['body']['form_url'] = $form_url;
            }
        }

        $responses = wp_remote_retrieve_body( wp_remote_request( $request_url, $request_args ) );

        //print_r(json_decode($responses,true)); die;

        /* If WP_Error, throw exception */
        if (is_wp_error($responses)) {
            //throw new Exception('Request failed. ' . $responses->get_error_messages());
        }

        $screen = $this->_generate_screen($responses, $this->current_screen);

        return $screen;
    }

    /**
     * @param string $data
     * @param $screen
     * @return string
     */
    private function _generate_screen($data = "", $screen)
    {
        $formData = json_decode($data, true);

        $error_code_a = ['503','404','409', '410', '411', '402','0'];

        if(isset($_GET["status"])){
            $screen = "error";

        }elseif(in_array($formData['code'], $error_code_a)){
            $screen = "error";
        }

        switch( $screen ) {

            case "profile":
                return $this->_profile_screen($formData);
                break;

            case "uploads":
                return $this->_uploads_screen($formData);
                break;

            case "amenities":
                return $this->_amenities_screen($formData);
                break;

            case "map":
                return $this->_map_screen($formData);
                break;

            case "mapview":
                return $this->_mapview_screen($formData);
                break;

            case "terms":
                return $this->_terms_screen($formData);
                break;

            case "signature":
                return $this->_signature_screen($formData);
                break;

            case "payment":
                return $this->_payment_screen($formData);
                break;

            case "error":
                return $this->_error_screen($formData);
                break;

            default:
                print_r($_SESSION);
                return "<p>Something went wrong.</p>";
        }
    }

    /**
     * @param int $map_id
     * @param string $pk
     * @return array|mixed|object
     */
    private function _get_map_data_by_id($map_id = 0, $pk = '')
    {
        global $wp_query;

        $attributes = json_decode($this->event_attributes, true);

        $headers = $this->_set_request_header(['Accept' => "application/json, text/javascript, */*; q=0.01", 'Authorization' => 'Basic ' . $this->plugin_api_auth_key]);

        $request_args = $this->_get_request_args([
                'method'  => 'GET',
                'headers' =>  $headers
            ]
        );

        // Dynamic form_key with url slug
        $attributes['id_hash'] = $wp_query->query_vars['id_hash'] ? $wp_query->query_vars['id_hash'] : 0;

        $request_url = $this->base_request_url . $attributes['id_hash'] . "/" . $attributes['event_a']['form_code'] . "/map/" . $pk . "/" . $map_id;

        $map_data = wp_remote_retrieve_body(wp_remote_request($request_url, $request_args));
        return json_decode($map_data, true);
    }

    /**
     * @param int $key
     * @param array $map
     * @return mixed
     */
    private function _get_area_data_by_id($key = 0, $map = [])
    {
        foreach($map['imagemap_areas_a'] as $areaKey => $area) {
            if($area['map_area_id'] == $key) {
                return $area;
            }
        }
    }

    /**
     * @param array $attributes
     * @return string
     */
    private function _get_paypal_data($attributes = [])
    {
        global $wp_query;

        if(isset($attributes['event_a']['form_code']) && isset($attributes['client_a']['profile_key'])) {

            $headers = $this->_set_request_header(['Accept' => "application/json, text/javascript, */*; q=0.01", 'Authorization' => 'Basic ' . $this->plugin_api_auth_key]);

            $request_args = $this->_get_request_args([
                    'method'  => 'POST',
                    'headers' =>  $headers,
                    'body' => ['return_url' => $this->_get_skip_url(['payment'=>'', 'return'=>1, 'pk'=>$attributes['applicant_a']['profile_key']])]
                ]
            );

            // Dynamic form_key with url slug
            $attributes['id_hash'] = $wp_query->query_vars['id_hash'] ? $wp_query->query_vars['id_hash'] : 0;

            $request_url = $this->base_request_url . $attributes['id_hash'] . "/" . $attributes['event_a']['form_code'] . "/". "payment" . "/" . $attributes['applicant_a']['profile_key'] . "/paypal";
            $response = wp_remote_retrieve_body(wp_remote_request($request_url, $request_args));

            return $response;
        }
    }

    /**
     * @param string $option
     * @param $formData
     * @return mixed|string
     */
    private function _get_data($option = "", $formData)
    {
        global $wp_query;

        if(isset($formData['event_a']['form_code']) && isset($formData['client_a']['profile_key'])) {

            $headers = $this->_set_request_header(['Accept' => "application/json, text/javascript, */*; q=0.01", 'Authorization' => 'Basic ' . $this->plugin_api_auth_key]);

            $request_args = $this->_get_request_args([
                    'method'  => 'GET',
                    'headers' =>  $headers
                ]
            );

            // Dynamic form_key with url slug
            $attributes['id_hash'] = $wp_query->query_vars['id_hash'] ? $wp_query->query_vars['id_hash'] : 0;

            $request_url = $this->base_request_url . $attributes['id_hash'] . "/" . $formData['event_a']['form_code'] . "/". $option . "/" . $formData['client_a']['profile_key'];

            $this->backup_attributes[$option] = wp_remote_retrieve_body(wp_remote_request($request_url, $request_args));

            return $this->backup_attributes[$option];
        }

        return json_encode([]);
    }

    /**
     * @param int $policy
     * @return string
     */
    private function _get_approval_status($approval_status = 0, $event_attributes_a) {

        $approval_status = $event_attributes_a['rec_type'] == 3 ? 99 : $approval_status;

        switch ($approval_status) {
            case 0:
                $status = '<p><h4>Application Status: <span class="green-alert">Pending Review</span></h4></p>';
                break;

            case 1:
                $status = '<p><h4>Application Status: <span class="green-alert">Approved</span></h4></p>';
                break;

            case 2:
                $status = '<p><h4>Application Status: On Wait-list</h4></p>';
                break;

            case 3:
                $status = '<p><h4>Application Status: <span class="red-alert">Rejected</span></h4></p>';
                break;

                default:
                break;
        }

        return $status;
        // return $event_attributes_a['rec_type'];
    }

    /**
     * @param int $policy
     * @return string
     */
    private function _display_payment_info($event_attributes_a) {
        ob_start();

        if($event_attributes_a['rec_type'] == 3){
            $cart_info_a = [
                "notice1_enable",
                "is_applic_deposit",
                "discount_scope",
                "discount_exp",
                "contact_email",
                "contact_name",
                "contact_phone"];
        }else{
            $cart_info_a = [
                "notice1_enable",
                "is_applic_deposit",
                "fee_structure",
                "fee_details",
                "discount_scope",
                "discount_exp",
                // "pay_policy",
                "fee_policy",
                "contact_email",
                "contact_name",
                "contact_phone"];
        }
        ?>

        <div style="width:100%">
            <div class="col-lg-offset-2" style="margin-top: 20px; margin-bottom: 5px;">
                <h3>Important Details <span class="red-alert">(Please Read)</span></h3>
            </div>
            <table style="width: 100%;" class="table table-striped table-bordered detail-view">
                <tbody>

                <?php
                foreach($cart_info_a AS $info_key){
                    $cld_label = ucwords(str_replace("_"," ",$info_key));
                    $cld_val = false;
                    $add_br = true;

                    switch ($info_key) {
                        case "notice1_enable":
                            $cld_label = "Notice";
                            if($event_attributes_a[$info_key] == 1) {
                                $cld_val = $event_attributes_a['notice1_body_merged'];
                                $add_br = false;
                            }else{
                                $cld_val = false;
                            }
                            break;

                        case "fee_policy":
                            $cld_label = "Service Fees";
                            $cld_val = $event_attributes_a[$info_key] == 1 ? "RegisterONE services fees will be applied to your total" : false;
                            break;

                        case "pay_policy":
                            if($event_attributes_a[$info_key] == 1) {
                                $cld_val = "Event payment is required AFTER you are approved.";
                            }elseif($event_attributes_a[$info_key] == 2) {
                                $cld_val = "You MAY make payment before application approval";
                            }elseif($event_attributes_a[$info_key] == 3) {
                                $cld_val = "Full payment MUST be made with the application before being approved";
                            }
                            break;

                        case "is_applic_deposit":
                            $cld_label = "Application Fee";
                            if($event_attributes_a["application_fee"] > 0) {
                                $cld_val = $event_attributes_a[$info_key] == 1 ? "Deposit is required. Deposits are subtracted from your remaining balance." : "Application fee is required";
                            }
                            break;

                        case "discount_scope":
                            // discount_type - 1 = percentage, 2 = absolute
                            $cld_label = "Discounts";

                            if($event_attributes_a['discount_scope'] == 1) {
                                $discount_scope = "Application Fee/Deposit";
                                $discount_percentage = (int)$event_attributes_a['discount_rate'];

                            }elseif($event_attributes_a['discount_scope'] == 2) {
                                $discount_scope = "Attendance Fee/Remaining Balance";
                                $discount_percentage = (int)$event_attributes_a['discount_rate'];

                            }elseif($event_attributes_a['discount_scope'] == 3) {
                                $discount_scope = "Application Fee and Remaining Balance";
                                $discount_percentage = (int)$event_attributes_a['discount_rate'];
                            }

                            $cld_val = isset($discount_percentage) ? "{$discount_percentage}% off of {$discount_scope}" : false;
                            $discount_exists = isset($discount_percentage) ? "{$discount_percentage}% off of {$discount_scope}" : false;
                            break;

                        case "discount_exp":
                            $cld_label = "Discount Deadline";
                            $cld_val = $event_attributes_a['discount_scope'] != 0 ? date_format(date_create($event_attributes_a[$info_key]), 'm/d/Y') . ' 11:59pm EST' : false;
                            break;

                        default;
                            $cld_val = $event_attributes_a[$info_key];
                            break;
                    }

                    if($cld_val){
                        $cld_val = $add_br ? nl2br($cld_val) : $cld_val;
                        ?>
                        <tr>
                            <th style="white-space: nowrap;"><?= $cld_label; ?></th>
                            <td style="text-align: left;"><?= $cld_val; ?></td>
                        </tr>
                        <?php
                    }
                }
                ?>
                </tbody>
            </table>
        </div>

        <?php
        return ob_get_clean();
    }

    /**
     * @param int $policy
     * @return string
     */
    private function _get_notice_by_pay_policy($policy = 1) {
        ob_start();
        switch ($policy) {
            case 1:
                ?>
                <div style="margin-bottom:5px; margin-top:5px;">
                    <h4>Full Payment Required <span style="text-decoration: underline;">after</span> application approval.</h4>
                </div>
                <?php
                break;
            case 2:
                ?>
                <div style="margin-bottom:5px; margin-top:5px;">
                    <h4>Full payment <span style="text-decoration: underline">encouraged</span> with application.</h4>
                </div>
                <?php
                break;
            case 3:
                ?>
                <div style="margin-bottom:5px; margin-top:5px;">
                    <h4>Full payment <span style="text-decoration: underline;">required</span> with application.</h4>
                </div>
                <?php
                break;
            default:
                break;
        }

        return ob_get_clean();
    }

    /**
     * handle form submissions
     */
    public function handle_form_submission()
    {
        if(empty($_POST) || !wp_verify_nonce($_POST['dwc_verify_submission'], 'dwc_form_submission')) {
            echo "Test failed! (handle_form_submission)";
        }else{
            // echo "<pre>";
            // print_r($_REQUEST);
            // print_r($this->_get_form_root_path(wp_get_referer()));
            // echo "</pre>";

            $this->_execute_form_submission($_REQUEST, $this->_get_form_root_path(wp_get_referer()));
        }
    }

    /**
     * handle map selection submission
     */
    public function handle_map_submission()
    {
        if(empty($_POST) || !wp_verify_nonce($_POST['dwc_verify_submission'], 'dwc_map_submission')) {
            echo "Test failed! (handle_map_submission)";
        }else{
            $this->_execute_form_submission($_REQUEST, "");
        }
    }

    /**
     * handle signature submission
     */
    public function handle_signature_submission()
    {
        if(empty($_POST) || !wp_verify_nonce($_POST['dwc_verify_submission'], 'dwc_signature_submission')) {
            echo "Test failed! (handle_signature_submission)";
        }else{
            $this->_execute_form_submission($_REQUEST, "");
        }
    }

    /**
     * @param array $args
     * @return array|string
     */
    private function _get_skip_url($args = [])
    {
        global $wp;
        // https://developer.wordpress.org/reference/functions/add_query_arg/
        // https://codex.wordpress.org/Function_Reference/home_url
        $skip_url = home_url(add_query_arg($args, $wp->request));
        if(count($args)) {
            $skip_url = explode("?", $skip_url);
            $skip_url[0] = rtrim($skip_url[0]) . '/';
            $skip_url = $skip_url[0] . '?' . $skip_url[1];
        } else {
            $skip_url = rtrim($skip_url) . '/';
        }
        $mt = str_replace('.', '', microtime(true));
        $skip_url .= "&v=".$mt;

        return $skip_url;
    }

    /**
     * @return mixed
     */
    private function _check_php_version()
    {
        return version_compare(PHP_VERSION, '5.5.0', '>=');
    }

    /**
     * @param string $path
     * @return mixed
     */
    private function _get_form_root_path($path = "")
    {
        $temp = explode("?", $path);
        return $temp[0];
    }

    /**
     * @param $data
     * @param string $root
     * @return bool|int
     */
    private function _execute_form_submission($data, $root = "")
    {
        switch($data['dwc_screen_name']) {

            case "profile":

                // print_r($data); //die;

                $responses = json_decode( $this->_execute_profile_screen_submission($data), true);

                // print_r($responses); die;

                if($responses['code'] == 201) {

                    $next_step_a = $responses['next_step'][0];
                    $key_pos = (int)array_search($data['dwc_screen_name'],$next_step_a);
                    $key_pos++;

                    $redirect_url = $root;
                    $redirect_url .= "?".$next_step_a[$key_pos];

                    if(isset($responses['profile_key']) && $responses['profile_key'] != '') {
                        $redirect_url .= "&pk=" . $responses['profile_key'];
                        $_SESSION['dwc_current_pk'] = $responses['profile_key'];
                    }

                    // print_r($responses); die;

                    wp_redirect($redirect_url);
                    exit();

                }elseif($responses['code'] == 0) {

                    $redirect_url = $root;
                    $redirect_url .= "?profile";

                    if(isset($responses['profile_key']) && $responses['profile_key'] != '') {
                        $redirect_url .= "&pk=" . $responses['profile_key'];
                        $_SESSION['dwc_current_pk'] = $responses['profile_key'];
                    }

                    $redirect_url .= "&status=" . $responses['status'];
                    wp_redirect($redirect_url);
                    exit();
                }
                break;
            
            case "uploads":

                $responses = $this->_execute_uploads_screen_submission($data);

                $next_step_a = $responses['upload_status']['next_step'][0];
                $key_pos = (int)array_search($data['dwc_screen_name'],$next_step_a);
                $key_pos++;

                $redirect_url = $root;
                $redirect_url .= "?".$next_step_a[$key_pos];

                if(isset($responses['profile_key']) && $responses['profile_key'] != '') {
                    $redirect_url .="&pk=" . $responses['profile_key'];
                    $_SESSION['dwc_current_pk'] = $responses['profile_key'];
                }

                wp_redirect($redirect_url);
                exit();
                break;

            case "amenities":

                $responses = json_decode($this->_execute_amenities_screen_submission($data), true);


                $redirect_url = $root;

                if($responses['code'] == 201 || $responses['code'] == 200) {
                    // Array ( [add_extra_fields] => Array ( [0] => 42 ) [status] => success [code] => 201 [detail] => Amenities updated )
                    if(isset($responses['add_extra_fields'])){
                        $extra_amenity_id = $responses['add_extra_fields'][0];
                        $redirect_url .= "?amenities&extra={$extra_amenity_id}";
                    }else {

                        $next_step_a = $responses['next_step'][0];
                        $key_pos = (int)array_search($data['dwc_screen_name'],$next_step_a);
                        $key_pos++;

                        $redirect_url = $root;
                        $redirect_url .= "?".$next_step_a[$key_pos];
                    }
                }else{
                    $redirect_url .= "?amenities";
                }

                if(isset($data['Clients']['Value']['applic_profile_key'] ) && $data['Clients']['Value']['applic_profile_key'] != '' ) {
                    $redirect_url .= "&pk=" . $data['Clients']['Value']['applic_profile_key'];
                    $_SESSION['dwc_current_pk'] = $data['Clients']['Value']['applic_profile_key'];
                }

                //echo $redirect_url;
                // print_r($responses); die;

                wp_redirect($redirect_url);
                exit();
                break;
            
            case "map":

                $this->_execute_map_screen_submission($data);
                break;
            
            case "signature":

                $this->_execute_signature_screen_submission($data);
                break;

            case "payment":

                $this->_execute_payment_screen_submission($data);

                $redirect_url = $root;
                $redirect_url .= "?payment";
                $redirect_url .= "&pk=" . $data['profile_key'];

                wp_redirect($redirect_url);
                exit();
                break;

            default:
                return 1;
        }

        return false;
    }

    /**
     * @param $data
     */
    private function _execute_map_screen_submission($data)
    {
        $headers = $this->_set_request_header(['Accept' => "application/json, text/javascript, */*; q=0.01", 'Authorization' => 'Basic ' . $this->plugin_api_auth_key]);

        $mapData = ['data' => $data['data']];
        $request_args = $this->_get_request_args([
                'method'  => 'POST',
                'headers' =>  $headers,
                'body'    => $mapData
            ]
        );

        $request_url = $this->base_request_url . $data['form_code'] . "/" . $data['dwc_screen_name'] . "/" . $data['profile_key'] . "/" . $data['map'];

        $responses = wp_remote_retrieve_body(wp_remote_request($request_url, $request_args));
        echo $responses;
        die();
    }

    /**
     * @param $data
     */
    private function _execute_signature_screen_submission($data) {

        $headers = $this->_set_request_header(['Accept' => "application/json, text/javascript, */*; q=0.01", 'Authorization' => 'Basic ' . $this->plugin_api_auth_key]);

        $signatureData = ['svg_xml' => $data['svg_xml'], 'svg' => $data['svg'], 'terms_scope' => $data['terms_scope']];
        $request_args = $this->_get_request_args([
                'method'  => 'POST',
                'headers' =>  $headers,
                'body'    => $signatureData
            ]
        );

        $request_url = $this->base_request_url . $data['form_code'] . "/" . $data['dwc_screen_name'] . "/" . $data['profile_key'];

        $responses = wp_remote_retrieve_body(wp_remote_request($request_url, $request_args));

        echo $responses;
        die();
    }

    /**
     * @param $data
     * @return string
     */
    private function _execute_profile_screen_submission($data)
    {
        global $wp;

        $headers = $this->_set_request_header(['Accept' => "application/json, text/javascript, */*; q=0.01", 'Authorization' => 'Basic ' . $this->plugin_api_auth_key]);

        if (count($data['Clients']['client_applic_type'])) {
            $data['Clients']['client_applic_type'] = json_encode($data['Clients']['client_applic_type']);
        }

        if (count($data['Clients']['client_merc'])) {
            $data['Clients']['client_merc'] = json_encode($data['Clients']['client_merc']);
        }

        $redirect_url = $_SERVER['HTTP_REFERER'];
        $data['Clients']['referring_url'] = json_encode($redirect_url);

        $request_args = $this->_get_request_args([
                'method' => 'POST',
                'headers' => $headers,
                'body' => $data['Clients']
            ]
        );

        if (isset($data['profile_key']) && $data['profile_key'] != '') {
            if (isset($data['rfc']) && $data['rfc'] != '') {
                $request_url = $this->base_request_url . $data['form_code'] . "/" . $data['dwc_screen_name'] . "/" . $data['profile_key'] . "/" . $data['rfc'];
            }else{
                $request_url = $this->base_request_url . $data['form_code'] . "/" . $data['dwc_screen_name'] . "/" . $data['profile_key'];
            }
        } else {
            if (isset($data['return_vendor'])) {
                $request_url = $this->base_request_url . $data['form_code'] . "/return/1";
            }else{
                $request_url = $this->base_request_url . $data['form_code'] . "/" . $data['dwc_screen_name'];
            }
        }

        //print_r($request_url); //die;
        //print_r($request_args); //die;

        $responses = wp_remote_retrieve_body(wp_remote_post($request_url, $request_args));

        //print_r($responses); die;

        return $responses;
    }

    /**
     * @param $data
     */
    private function _execute_payment_screen_submission($data) {

        $headers = $this->_set_request_header(['Accept' => "application/json, text/javascript, */*; q=0.01", 'Authorization' => 'Basic ' . $this->plugin_api_auth_key]);

        // $signatureData = ['svg_xml' => $data['svg_xml'], 'svg' => $data['svg'], 'terms_scope' => $data['terms_scope']];
        $request_args = $this->_get_request_args([
                'method'  => 'POST',
                'headers' =>  $headers,
                'body'    => $data
            ]
        );

        $request_url = $this->base_request_url . $data['form_code'] . "/" . $data['dwc_screen_name'] . "/" . $data['profile_key'] . "/authnet";

        // echo $request_url; die;
        // http://api.registerone.net/v1/apply//payment/
        // http://api.registerone.net/v1/apply/eyq7ywrx/payment/
        // http://api.registerone.net/v1/apply/eyq7ywrx/amenities/faTtXrSG5LMlNvebuk3G

        $responses = wp_remote_retrieve_body(wp_remote_request($request_url, $request_args));

        return $responses;
        // die();
    }

    /**
     * @param $data
     * @return string
     */
    private function _execute_amenities_screen_submission($data)
    {
        // print_r($data); die;

        $headers = $this->_set_request_header(['Accept' => "application/json, text/javascript, */*; q=0.01", 'Authorization' => 'Basic ' . $this->plugin_api_auth_key]);

        // bug fix to correct issue when de-selecting All amenities
        // $data['Clients']['amenity_types'] may not be set
        $data['Clients']['amenity_types'] = isset($data['Clients']['amenity_types']) ? $data['Clients']['amenity_types'] : 0;

        if(isset($data['amenity_extra'])){
            $body = ['AmenitiesExtra' => ['amenity_extra' => $data['amenity_extra']]];
        }else{
            $body = ['Amenities' => ['amenity_types' => $data['Clients']['amenity_types']], 'amenity_qty' => $data['amenity_qty']];
        }

        $request_args = $this->_get_request_args([
                'method'  => 'POST',
                'headers' =>  $headers,
                'body'    => $body
            ]
        );

        if(isset($data['Clients']['Value']['applic_profile_key']) && $data['Clients']['Value']['applic_profile_key'] != '') {
            $request_url = $this->base_request_url . $data['Clients']['Value']['form_code'] . '/' . $data['dwc_screen_name'] . "/" . $data['Clients']['Value']['applic_profile_key'];
        }else{
            $request_url = $this->base_request_url . $data['Clients']['Value']['form_code'] . "/" . $data['dwc_screen_name'];
        }

        // echo $request_url; die;
        // http://api.registerone.net/v1/apply/eyq7ywrx/amenities/faTtXrSG5LMlNvebuk3G

        $responses = wp_remote_retrieve_body(wp_remote_request($request_url, $request_args));

        if(isset($data['amenity_extra'])) {
            //print_r($responses);
            //die;
        }

        return $responses;
    }

    /**
     * @param $data
     * @return array
     */
    private function _execute_uploads_screen_submission($data)
    {
        $postData = [];
        if(isset($_FILES['DynamicModel']['name']) && count($_FILES['DynamicModel']['name'])) {

            foreach ($_FILES['DynamicModel']['name'] as $key => $file) {

                if($file != '') {
                    if($this->_check_php_version()) {
                        $postData[$key] = new CURLFile($_FILES['DynamicModel']['tmp_name'][$key], $_FILES['DynamicModel']['type'][$key], basename($_FILES['DynamicModel']['name'][$key]));
                    }else{
                        $responses[$key] = '@' . $_FILES['DynamicModel']['tmp_name'][$key] . ';filename=' . $_FILES['DynamicModel']['name'][$key] . ';type=' . $_FILES['DynamicModel']['type'][$key];
                    }
                }
            }
        }

        $postData[] = ["upload_files" => ""];

        $postData['meta'] = $_POST['DynamicModel']['meta'];

        if(isset($data['DynamicModel']['Value']['applic_profile_key']) && $data['DynamicModel']['Value']['applic_profile_key'] != '') {
            $request_url = $this->base_request_url . $data['form_code'] . '/' . $data['dwc_screen_name'] . "/" . $data['DynamicModel']['Value']['applic_profile_key'];
        }else{
            $request_url = $this->base_request_url . $data['form_code'] . "/" . $data['dwc_screen_name'];
        }

        // initialize the curl request
        // needed for file uploads
        $curl_request = curl_init();

        curl_setopt($curl_request, CURLOPT_URL, $request_url);
        // modify curl request to send file
        curl_setopt($curl_request, CURLOPT_POST, true);

        // add file into curl request
        curl_setopt($curl_request, CURLOPT_POSTFIELDS, $this->_build_post_fields($postData));

        // initialize curl headers
        $curl_headers = $this->_set_request_header([
            'Content-Type' => "multipart/form-data",
            'Accept' => "application/json, text/javascript, */*; q=0.01",
            'Authorization' => 'Basic ' . $this->plugin_api_auth_key
        ]);

        // set curl http headers
        curl_setopt($curl_request, CURLOPT_HTTPHEADER, $curl_headers);

        // modify curl to return response
        curl_setopt($curl_request, CURLOPT_RETURNTRANSFER, true);

        $curl_responses = curl_exec($curl_request);

        curl_close($curl_request);

        // die(json_decode($curl_responses, true));

        // set the response
        $responses = [];
        if(isset($data['DynamicModel']['Value']['applic_profile_key']) && $data['DynamicModel']['Value']['applic_profile_key']) {
            $responses['profile_key'] = $data['DynamicModel']['Value']['applic_profile_key'];
            $responses['upload_status'] = json_decode($curl_responses, true);
        }

        return $responses;
    }

    /**
     * Converts multidimensional arrays to single dim array for CURLOPT_POSTFIELDS
     * @param $data
     * @param string $existingKeys
     * @param array $returnArray
     * @return array
     */
    private function _build_post_fields( $data,$existingKeys='',&$returnArray=[]){
        if(($data instanceof CURLFile) or !(is_array($data) or is_object($data))){
            $returnArray[$existingKeys]=$data;

            return $returnArray;

        }else{
            foreach ($data as $key => $item) {
                $this->_build_post_fields($item,$existingKeys?$existingKeys."[$key]":$key,$returnArray);
            }
            return $returnArray;
        }
    }

    /**
     * @param array $args
     * @return array
     */
    private function _get_request_args($args = [])
    {
        return $this->_set_request_args($args);
    }

    /**
     * @param array $headers
     * @return array
     */
    private function _set_request_header($headers = [])
    {
        $current_headers = $this->default_headers;

        if(!empty($current_headers)) {
            foreach($headers as $key => $val) {

                $current_headers[$key] = $val;
            }
        }
        return $this->_generate_request_headers($current_headers);
    }

    /**
     * @param array $headers
     * @return array
     */
    private function _generate_request_headers($headers = [])
    {
        $request_headers = [];

        if(!empty($headers)) {
            foreach($headers as $key => $val) {
                $request_headers[] = $key . ":" . $val;
            }
        }
        return $request_headers;
    }

    /**
     * @param array $args
     * @return array
     */
    private function _set_request_args($args = [])
    {
        $current_args = $this->default_args;

        if(!empty($args)) {

            foreach($args as $key => $val) {
                if(array_key_exists($key, $current_args)) {
                    $current_args[$key] = $val;
                }
            }
        }
        return $current_args;
    }

    /**
     * @return string
     */
    private function _get_current_screen()
    {
        if(isset($_GET['css'])){
            return "css";

        }elseif(isset($_GET['profile'])){
            return "profile";
            
        }elseif(isset($_GET['uploads'])){
            return "uploads";
            
        }elseif(isset($_GET['amenities'])) {
            if(isset($_GET['map']) && $_GET['map'] != '') {
                return "map";
            }else{
                return "amenities";
            }
            
        }elseif(isset($_GET['terms'])) {
            if(isset($_GET['signature']) && $_GET['signature'] != '') {
                return "signature";
            }else{
                return "terms";
            }
            
        }elseif(isset($_GET['payment'])){
            return "payment";

        }elseif(isset($_GET['mapview'])){
            return "mapview";

        }else{
            return "profile";
        }
    }

    /**
     * @param $formData
     * @return string
     */
    private function _payment_screen($formData)
    {
        $form = "";

        $form = $this->_set_application_title($this->event_attributes, $form);

        $form = $this->_set_application_steps("payment", $form, $formData);

        $form = $this->_create_payment_screen_fields($formData, $form);

        return $this->_wrap_with_parents($form);
    }

    /**
     * @param $formData
     * @return string
     */
    private  function _signature_screen($formData)
    {
        $form = "";

        $form = $this->_create_signature_screen_fields($formData, $form);

        return $form;
    }

    /**
     * @param $formData
     * @return string
     */
    private function _terms_screen($formData)
    {
        $form = "";

        $form = $this->_set_application_title($this->event_attributes, $form);

        $form = $this->_set_application_steps("terms", $form, $formData);

        $form = $this->_create_terms_screen_fields($formData, $form);

        return $this->_wrap_with_parents($form);
    }

    /**
     * @param $formData
     * @return string
     */
    private function _map_screen($formData)
    {
        $form = "";

        $form = $this->_create_map_screen_fields($formData, $form);

        return $form;
    }


    /**
     * @param $formData
     * @return string
     */
    private function _mapview_screen($formData)
    {
        $form = "";

        $form = $this->_create_mapview_screen_fields($formData, $form);

        return $form;
    }


    /**
     * @param $formData
     * @return string
     */
    private function _amenities_screen($formData)
    {
        $form = "";

        $form = $this->_set_application_title($this->event_attributes, $form);

        $form = $this->_set_application_steps("amenities", $form, $formData);

        if(isset($_GET['extra']) && $_GET['extra'] != null){
            $form = $this->_create_extra_screen_fields($formData, $form);
            // die("badge");
        }else{
            $form = $this->_create_amenities_screen_fields($formData, $form);
        }

        return $this->_wrap_with_parents($form);
    }

    /**
     * @param $formData
     * @return string
     */
    private function _uploads_screen($formData)
    {
        $form = "";

        $form = $this->_set_application_title($this->event_attributes, $form);

        $form = $this->_set_application_steps('uploads', $form, $formData);

        $form = $this->_create_uploads_screen_fields($formData, $form);

        return $this->_wrap_with_parents($form);
    }

    /**
     * @param $formData
     * @return string
     */
    private function _error_screen($formData)
    {
        $form = "";

        $form = $this->_set_application_title($this->event_attributes, $form);

        $form = $this->_set_application_steps('profile', $form, $formData);

        $form = $this->_create_error_screen($formData, $form);

        return $this->_wrap_with_parents($form);
    }

    /**
     * @param $formData
     * @return string
     */
    private function _profile_screen($formData)
    {
        $form = "";

        $form = $this->_set_application_title($this->event_attributes, $form);

        $form = $this->_set_application_steps('profile', $form, $formData);

        $form = $this->_create_profile_screen_fields($formData, $form);

        return $this->_wrap_with_parents($form);
    }

    /**
     * @param $formData
     * @param $field_type (checkbox/radio)
     * @return string
     */
    private function _generate_checkboxes_client_types($input_data, $field_type = 'checkbox') {

        $client_types_description_a = $input_data['client_types_description_a'];

        $applicant_a = [];
        if(isset($input_data['client_type']))
            $applicant_a = json_decode($input_data['client_type'], true);

        $selectors = '';
        if(isset($input_data['client_type']) && !empty($applicant_a)) {
            $c = 0;
            foreach($input_data['client_types_a'] as $tKey => $tVal) {
                $c++;
                $div = "<div class=\"col-lg-8 {$field_type}\">";

                if($field_type == 'checkbox') {
                    $required = $c == 1 && $input_data['required'] == 1 ? 'data-parsley-mincheck="1" data-parsley-errors-container="#error-box-' . $input_data['model_field'] . '" required' : '';
                }else{
                    $required = $c == 1 && $input_data['required'] == 1 ? 'data-parsley-errors-container="#error-box-' . $input_data['model_field'] . '" required' : '';
                }

                if(in_array($tKey, $applicant_a)) {
                    $div .= "<input ".$required." id=\"client_type_" . $tKey . "\" class=\"client_type\" type=\"{$field_type}\" checked=\"checked\" name=\"Clients[client_applic_type][]\" value=\"" . $tKey . "\">";
                }else{
                    $div .= "<input ".$required." id=\"client_type_" . $tKey . "\" class=\"client_type\" type=\"{$field_type}\" name=\"Clients[client_applic_type][]\" value=\"" . $tKey . "\">";
                }

                $div .= "<label for=\"client_type_" . $tKey. "\">{$tVal}</label>";
                if(!empty($client_types_description_a[$tKey])){
                    $div .= "<div class='well well-infobox client_desc'>{$client_types_description_a[$tKey]}</div>";
                }
                $div .= "</div>";

                $selectors .= $div;
            }
        }else{
            $c = 0;
            foreach($input_data['client_types_a'] as $tKey => $tVal) {
                $c++;
                $div = "<div class=\"col-lg-8 {$field_type}\">";

                $required = $c == 1 && $input_data['required'] == 1 ? 'data-parsley-errors-container="#error-box-'.$input_data['model_field'].'" required' : '';

                $div .= "<input ".$required." id=\"client_type_" . $tKey . "\" class=\"client_type\" type=\"{$field_type}\" name=\"Clients[client_applic_type][]\" value=\"" . $tKey. "\">";

                $div .= "<label for=\"client_type_" . $tKey. "\">{$tVal}</label>";
                if(!empty($client_types_description_a[$tKey])){
                    $div .= "<div class='well well-infobox client_desc'>{$client_types_description_a[$tKey]}</div>";
                }
                $div .= "</div>";

                $selectors .= $div;
            }
        }
        return $selectors;
    }

    /**
     * @param $key
     * @param $data
     * @param bool $partial
     * @return string
     */
    private function _uploaded_file_status($key, $data, $partial = false)
    {
        ob_start();
        if(isset($data['lic_id_status_a'][$key])):
        ?>
            <?php if(!$partial): ?>
            <span style="font-size: 17px;" class="green-alert">
                <i class="fa fa-check" aria-hidden="true"></i>
            </span>
            Uploaded <strong><?php echo $data['lic_id_file_names_a'][$key]; ?></strong>
            <?php endif; ?>
        <?php if($data['lic_id_status_a'][$key] == 1): ?>
            <span class="tooltip-success label label-success" data-original-title="" style="white-space: nowrap; text-decoration: none; cursor: default;">Approved.</span>
        <?php elseif($data['lic_id_status_a'][$key] == 2): ?>
            <span class="tooltip-danger label label-danger" data-original-title="" style="white-space: nowrap; text-decoration: none; cursor: default;">On Hold, please review.</span>
        <?php else: ?>
            <span class="tooltip-warning label label-warning" data-original-title="" style="white-space: nowrap; text-decoration: none; cursor: default;">Pending Review.</span>
        <?php
            endif;
        endif;

        return ob_get_clean();
    }

    /**
     * @param array $area
     * @return string
     */
    private function _generate_viewmap_area($area = [])
    {
        ob_start();
        ?>
        <area id="id_<?php echo $area['href']; ?>" key="id_<?php echo $area['href']; ?>" full="#<?php echo $area['href']; ?>" title="<?php echo $area['title']; ?>" shape="<?php echo $area['type']; ?>" coords="<?php echo $area['coords']; ?>" href="#<?php echo $area['href']; ?>" alt="<?php echo $area['alt']; ?>">
        <?php
        return ob_get_clean();
    }

    /**
     * @param array $area
     * @return string
     */
    private function _generate_map_area($area = [])
    {
        ob_start();
        ?>
        <area key="<?php echo $area['href']; ?>" full="#<?php echo $area['href']; ?>" title="<?php echo $area['title']; ?>" shape="<?php echo $area['type']; ?>" coords="<?php echo $area['coords']; ?>" href="#<?php echo $area['href']; ?>" alt="<?php echo $area['alt']; ?>">
        <?php
        return ob_get_clean();
    }

    /**
     * @param $formData
     * @param $form
     * @return string
     */
    private function _create_payment_screen_fields($formData, $form)
    {
        global $wp;
        $current_url = home_url(add_query_arg([], $wp->request));

        $event_attributes = json_decode($this->event_attributes, true);
        $applicant_payment = json_decode($this->_get_data("payment", $event_attributes), true);
        // $applicant_signature = json_decode($this->_get_data("signature", $event_attributes), true);

        $event_attributes['event_a']['notice1_body_merged'] = $applicant_payment['notice1_body_merged'];

        $approval_status = $event_attributes['applicant_a']['approval_status'];
        $request_url = $this->base_request_url . $event_attributes['id_hash'] . "/";

        $paypap_notice = '';
        if(isset($_GET['return']) && $_GET['return'] != null){
           $paypap_notice = '<div class="well well-sm" style="margin-bottom:10px; margin-top:10px;"><i style="float:left; margin-right:10px;" class="fa fa-2x fa-exclamation-circle blue" aria-hidden="true"></i>
                Payments may take a few seconds to register as paid, so refresh this page if your payment still displays as due.
            </div>';
        }

        ob_start();

        // fallback for older plugins
        // to-do: make fallback function to instantiate default vals from api to support older plugins
        $applicant_payment['fee_override'] = array_key_exists('fee_override',$applicant_payment) ? $applicant_payment['fee_override'] : 0;

        if(isset($applicant_payment['dataProvider_fees']['allModels'])){
            if(count($applicant_payment['dataProvider_fees']['allModels'])){
                $cart_items_exists = true;
            }
        }else{
            $cart_items_exists = false;
        }

        if(!$cart_items_exists &&
           $applicant_payment['fee_total'] == 0 &&
           $applicant_payment['fee_total_paid'] == 0 &&
           $applicant_payment['fee_override'] == 0 &&
           $applicant_payment['fee_total_once_approved'] == 0):

            if(count($event_attributes['error_details_a'])):

                if (in_array('map', $event_attributes['error_details_a'])): ?>
                    <a class="btn btn-error"
                       href="<?php echo $this->_get_skip_url(['amenities' => '', 'pk' => $_GET['pk']]); ?>"
                       style="text-decoration: none;">
                        <h3>Select a booth location</h3>
                    </a>
                <?php endif; ?>

                <?php if (in_array('uploads', $event_attributes['error_details_a'])): ?>
                    <a class="btn btn-error"
                       href="<?php echo $this->_get_skip_url(['uploads' => '', 'pk' => $_GET['pk']]); ?>"
                       style="text-decoration: none;">
                        <h3>Please upload required docs</h3>
                    </a>
                <?php endif; ?>

                <?php if (in_array('terms', $event_attributes['error_details_a'])): ?>
                    <a class="btn btn-error"
                       href="<?php echo $this->_get_skip_url(['terms' => '', 'pk' => $_GET['pk']]); ?>"
                       style="text-decoration: none;">
                        <h3>Signature required</h3>
                    </a>
                <?php endif; ?>

            <?php else: ?>

                <div style="margin-top:20px; width:100%; text-align:center;">
                    <p><h2><span class="green-alert"><i class="fa fa-check" aria-hidden="true"></i> Application Complete.</span></h2></p>
                </div>

                <div style="width:100%; text-align:center;">
                    <?php echo $this->_get_approval_status($approval_status, $event_attributes['event_a']); ?>
                    <?php echo $this->_display_payment_info($event_attributes['event_a']); ?>
                </div>

            <?php endif; ?>

        <?php else: ?>

            <!--// begin cart -->
            <div class="grid-view">
                <table class="table mce-table-striped table-bordered">
                    <thead>
                        <tr>
                            <th>Item Description</th>
                            <th>Status</th>
                            <th>Qty</th>
                            <th>Fee</th>
                            <th>Sub Total</th>
                            <th>Qty Purchased**</th>
                        </tr>
                    </thead>
                    <tbody>
                        <?php if(isset($applicant_payment['dataProvider_fees']['allModels'])): ?>
                            <?php if(count($applicant_payment['dataProvider_fees']['allModels'])): ?>
                                <?php foreach($applicant_payment['dataProvider_fees']['allModels'] as $key => $product): ?>
                                    <tr data-key="<?php echo $product['id']; ?>">
                                        <td><?php echo $product['item_desc']; ?></td>
                                        <td><?php echo $product['status']; ?></td>
                                        <td><?php echo $product['qty']; ?></td>
                                        <td><?php echo $product['fee']; ?></td>
                                        <td><?php echo $product['fee_sub_total']; ?></td>
                                        <td><?php echo $product['qty_paid']; ?></td>
                                    </tr>
                                <?php endforeach; ?>
                            <?php endif;?>
                        <?php endif; ?>
                    </tbody>
                </table>
            </div>

            <h6>**this is the quantity you have <span style="text-decoration: underline;">already</span> paid for.</h6>
            <!--// end cart -->

            <?= $paypap_notice; ?>

            <?php

            if(count($event_attributes['error_details_a'])):

                if (in_array('map', $event_attributes['error_details_a'])):
                $app_incomplete = true;
                ?>
                    <a class="btn btn-error"
                       href="<?php echo $this->_get_skip_url(['amenities' => '', 'pk' => $_GET['pk']]); ?>"
                       style="text-decoration: none;">
                        <h3>Select a booth location</h3>
                    </a>
                <?php endif; ?>

                <?php if(in_array('uploads', $event_attributes['error_details_a'])):
                $app_incomplete = true;
                ?>
                    <a class="btn btn-error"
                       href="<?php echo $this->_get_skip_url(['uploads' => '', 'pk' => $_GET['pk']]); ?>"
                       style="text-decoration: none;">
                        <h3>Please upload required docs</h3>
                    </a>
                <?php endif; ?>

                <?php if(in_array('terms', $event_attributes['error_details_a'])):
                $app_incomplete = true;
                ?>
                    <a class="btn btn-error"
                       href="<?php echo $this->_get_skip_url(['terms' => '', 'pk' => $_GET['pk']]); ?>"
                       style="text-decoration: none;">
                        <h3>Signature required</h3>
                    </a>
                <?php endif; ?>

            <?php endif; ?>

            <?php if($applicant_payment['fee_total'] == 0 && $applicant_payment['fee_total_paid'] > 0 && !$app_incomplete): ?>

                <div style="margin-top:20px; width:100%; text-align:center;">
                    <h2><span class="green-alert"><i class="fa fa-check" aria-hidden="true"></i> Application Complete.</span></h2>
                </div>

            <div style="width:100%; text-align:center;">
                    <?php echo $this->_get_approval_status($approval_status, $event_attributes['event_a']); ?>
                    <?= $this->_display_payment_info($event_attributes['event_a']); ?>
                </div>

            <?php elseif(!$app_incomplete): ?>

                <?php if(isset($applicant_payment['fee_total'])):

                    if ($applicant_payment['fee_total'] > 0):

                        if (isset($applicant_payment['pay_options'])):

                            /*
                             * pay_policy
                             * 1 - block until approved
                             * 2 - may pay before approval
                             * 3 - must pay before approval
                             * */

                            //////////////////////////////
                            // payments due and info block
                            ?>

                            <div style="margin-top: 10px;">
                                <?php echo $this->_get_approval_status($approval_status, $event_attributes['event_a']); ?>
                                <?php echo "<h4>Payment Due: <strong>\$".number_format($applicant_payment['fee_total'],2)."</strong></h4>"; ?>
                                <h4>Payment deadline:

                                <?php if($event_attributes['event_a']['pay_policy'] == 3): ?>
                                    <span class='red-alert'>With Application</span>

                                <?php else: ?>
                                    <strong>
                                        <?php
                                        $date = $event_attributes['event_a']['att_fee_deadline'] == '1969-12-31' ? 'not set' : date_format(date_create($event_attributes['event_a']['att_fee_deadline']), 'm/d/Y');
                                        ?>
                                        <?= $date; ?>
                                    </strong>
                                <?php endif; ?>
                                </h4>

                                <?php
                                    $display_discount = false;
                                    if($event_attributes['event_a']['discount_scope'] == 1) {
                                        $display_discount = true;
                                        $discount_scope = "Application Fee/Deposit";
                                        $discount_percentage = (int)$event_attributes['event_a']['discount_rate'];

                                    }elseif($event_attributes['event_a']['discount_scope'] == 2) {
                                        $display_discount = true;
                                        $discount_scope = "Attendance Fee/Remaining Balance";
                                        $discount_percentage = (int)$event_attributes['event_a']['discount_rate'];

                                    }elseif($event_attributes['event_a']['discount_scope'] == 3) {
                                        $display_discount = true;
                                        $discount_scope = "All fees";
                                        $discount_percentage = (int)$event_attributes['event_a']['discount_rate'];
                                    }

                                    $discount_exp = $event_attributes['event_a']['discount_scope'] != 0 ? date_format(date_create($event_attributes['event_a']['discount_exp']), 'm/d/Y') . ' 11:59pm EST' : false;
                                ?>

                                <?php if($display_discount): ?>
                                    <h4>Cart Discount: <span class='green-alert'><?= $discount_percentage; ?>% (<?= $discount_scope; ?>)</span></h4>
                                    <h4>If You Pay By: <span class='red-alert'><?= $discount_exp; ?></span></h4>
                                <?php endif; ?>


                                <div style="padding-top:10px;">
                                    <?php if(in_array('pp_b', $applicant_payment['pay_options'])): ?>
                                        <img style="width:275px height:48px;"
                                             src="<?php echo REGISTERONE_WORDPRESS_CLIENT_URL; ?>/front-end/assets/img/paypal-img1-footer-275.png"
                                             alt="paypal">
                                    <?php endif; ?>
                                </div>

                            <?php

                            ////////////////////////////////////////////
                            // BEGIN - generate paypal button and encrypted form
                            if (in_array('pp_b', $applicant_payment['pay_options'])):

                                $paypal_data = json_decode($this->_get_paypal_data($event_attributes), true);

                                if($paypal_data['encrypted_form_vals']): ?>

                                <form action="https://www.paypal.com/cgi-bin/webscr" method="post">
                                    <input type="hidden" name="cmd" value="_s-xclick">
                                    <input type="hidden" name="encrypted"
                                           value="<?php echo $paypal_data['encrypted_form_vals']; ?>">
                                    <div class="form-group">
                                        <button type="submit" class="btn btn-success"><i class="fa fa-lock" style="font-size:20px"></i> Pay using Paypal</button>
                                        <div id="submit-spinner" class="pull-left"></div>
                                    </div>
                                </form>

                                <?php //else: ?>
                                <!--div class="well well-sm" style="margin-bottom:10px; margin-top:10px;"><i style="float:left; margin-right:10px; color:red;" class="fa fa-2x fa-exclamation-circle" aria-hidden="true"></i>
                                    Paypal not configured correctly.
                                </div-->
                                <?php endif; ?>

                            <?php elseif (in_array('auth_net', $applicant_payment['pay_options'])): ?>

                                <div class="cart_box cart_row">
                                    <div class="col-100">

                                        <?php if($applicant_payment['authnet_sandbox'] == 1): ?>
                                            <div style="color:red; width:100%; margin-bottom:10px;">
                                                RUNNING IN SANDBOX MODE!
                                            </div>
                                        <?php endif; ?>

                                        <form data-parsley-validate="" id="clients-form" class="form-horizontal" name="step-5" action="<?php echo admin_url('admin-ajax.php'); ?>" method="post" enctype="multipart/form-data">

                                        <input type="hidden" name="_csrf" value="">
                                        <input type="hidden" name="dwc_screen_name" value="payment">
                                        <?php wp_nonce_field('dwc_form_submission', 'dwc_verify_submission'); ?>
                                        <input type="hidden" name="action" value="dwc_form_submission">

                                        <div class="cart_row">
                                            <div class="col-50">
                                                <h3>Billing Address</h3>

                                                <div class="cart_row">
                                                    <div class="col-50">
                                                        <label class="control-label-cart" for="fname">First Name</label>
                                                        <input type="text" id="fname" name="billingFirstName" placeholder="Joe" class="form-control">
                                                    </div>
                                                    <div class="col-50">
                                                        <label class="control-label-cart" for="lname">Last Name</label>
                                                        <input type="text" id="lname" name="billingLastName" placeholder="Smith" class="form-control">
                                                    </div>
                                                </div>

                                                <label class="control-label-cart" for="email"><i class="fa fa-envelope"></i> Email</label>
                                                <input type="text" id="email" name="email" placeholder="john@example.com" class="form-control">

                                                <label class="control-label-cart" for="phone"><i class="fa fa-phone"></i> Phone</label>
                                                <input type="text" id="phone" name="phoneNumber" placeholder="513-123-4567" class="form-control">

                                                <label class="control-label-cart" for="adr"><i class="fa fa-address-card-o"></i> Address</label>
                                                <input type="text" id="adr" name="billingAddress" placeholder="542 W. 15th Street" class="form-control">

                                                <label class="control-label-cart" for="city"><i class="fa fa-institution"></i> City</label>
                                                <input type="text" id="city" name="billingCity" placeholder="New York" class="form-control">


                                                <div class="cart_row" style="padding-left:15px; padding-right:15px">
                                                    <div class="col-33">
                                                        <label class="control-label-cart" for="state">State</label>
                                                        <select type="text" id="state" class="form-control" name="billingState"><option value=""></option><option value="Alabama">Alabama</option><option value="Alaska">Alaska</option><option value="Arizona">Arizona</option><option value="Arkansas">Arkansas</option><option value="California">California</option><option value="Colorado">Colorado</option><option value="Connecticut">Connecticut</option><option value="Delaware">Delaware</option><option value="District of Columbia">District of Columbia</option><option value="Florida">Florida</option><option value="Georgia">Georgia</option><option value="Hawaii">Hawaii</option><option value="Idaho">Idaho</option><option value="Illinois">Illinois</option><option value="Indiana">Indiana</option><option value="Iowa">Iowa</option><option value="Kansas">Kansas</option><option value="Kentucky">Kentucky</option><option value="Louisiana">Louisiana</option><option value="Maine">Maine</option><option value="Maryland">Maryland</option><option value="Massachusetts">Massachusetts</option><option value="Michigan">Michigan</option><option value="Minnesota">Minnesota</option><option value="Mississippi">Mississippi</option><option value="Missouri">Missouri</option><option value="Montana">Montana</option><option value="Nebraska">Nebraska</option><option value="Nevada">Nevada</option><option value="New Hampshire">New Hampshire</option><option value="New Jersey">New Jersey</option><option value="New Mexico">New Mexico</option><option value="New York">New York</option><option value="North Carolina">North Carolina</option><option value="North Dakota">North Dakota</option><option value="Ohio" selected="selected">Ohio</option><option value="Oklahoma">Oklahoma</option><option value="Oregon">Oregon</option><option value="Pennsylvania">Pennsylvania</option><option value="Rhode Island">Rhode Island</option><option value="South Carolina">South Carolina</option><option value="South Dakota">South Dakota</option><option value="Tennessee">Tennessee</option><option value="Texas">Texas</option><option value="Utah">Utah</option><option value="Vermont">Vermont</option><option value="Virginia">Virginia</option><option value="Washington">Washington</option><option value="West Virginia">West Virginia</option><option value="Wisconsin">Wisconsin</option><option value="Wyoming">Wyoming</option><option value="Armed Forces Americas">Armed Forces Americas</option><option value="Armed Forces Europe">Armed Forces Europe</option><option value="Armed Forces Pacific">Armed Forces Pacific</option></select>

                                                    </div>
                                                    <div class="col-33">
                                                        <label class="control-label-cart" for="zip">Zip</label>
                                                        <input style="width:70px" type="text" id="zip" name="billingZip" placeholder="10001" class="form-control">
                                                    </div>
                                                    <div class="col-33">
                                                        <label class="control-label-cart" for="country">Country</label>
                                                        <select type="text" id="country" class="form-control" name="billingCountry"><option value="USA">USA</option></select>
                                                    </div>
                                                </div>
                                            </div>

                                            <div class="col-50">
                                                <div class="icon-container">
                                                    <img class="" style="width:224px height:119px;"
                                                         src="<?= REGISTERONE_WORDPRESS_CLIENT_URL; ?>/front-end/assets/img/authorize.png"
                                                         alt="Payments through Authorize.net">
                                                </div>

                                                <h3>Payment</h3>
                                                <label class="control-label-cart" for="cname">Name on Card</label>
                                                <input type="text" id="cname" name="billingCompany" placeholder="John More Doe" class="form-control">

                                                <label class="control-label-cart" for="ccnum">Credit card number</label>
                                                <input type="text" id="ccnum" name="cardNumber" placeholder="1111-2222-3333-4444" class="form-control">

                                                <div class="cart_row">
                                                    <div class="col-50">
                                                        <label class="control-label-cart" for="expmonth">Exp Month</label>
                                                        <select type="text" id="expmonth" class="form-control" name="ddMonths">
                                                            <option value=""></option>
                                                            <option value="01">January (01)</option>
                                                            <option value="02">February (02)</option>
                                                            <option value="03">March (03)</option>
                                                            <option value="04">April (04)</option>
                                                            <option value="05">May (05)</option>
                                                            <option value="06">June (06)</option>
                                                            <option value="07">July (07)</option>
                                                            <option value="08">August (08)</option>
                                                            <option value="09">September (09)</option>
                                                            <option value="10">October (10)</option>
                                                            <option value="11">November (11)</option>
                                                            <option value="12">December (12)</option>
                                                        </select>
                                                    </div>

                                                    <div class="col-50">
                                                        <label class="control-label-cart" for="expyear">Exp Year</label>
                                                        <select type="text" id="expyear" class="form-control" name="ddYears">
                                                            <option value=""></option>
                                                            <option value="2015">2015</option>
                                                            <option value="2016">2016</option>
                                                            <option value="2017">2017</option>
                                                            <option value="2018">2018</option>
                                                            <option value="2019">2019</option>
                                                            <option value="2020">2020</option>
                                                            <option value="2021">2021</option>
                                                            <option value="2022">2022</option>
                                                            <option value="2023">2023</option>
                                                            <option value="2024">2024</option>
                                                        </select>

                                                    </div>
                                                </div>
                                            </div>
                                        </div>

                                            <div class="cart_left">
                                                <input type="hidden" class="form-control" name="cart_code" value="<?= $applicant_payment['cart_code'] ?>">

                                                <input type="hidden" id="payment-client_profile_key" class="form-control" name="client_profile_key" value="<?= $event_attributes['client_a']['profile_key']; ?>">

                                                <input type="hidden" id="payment-applic_profile_key" class="form-control" name="profile_key" value="<?= $event_attributes['applicant_a']['profile_key']; ?>">

                                                <input type="hidden" id="payment-apply_step5" class="form-control" name="apply_step5" value="1">

                                                <input type="hidden" id="payment-event-code" class="form-control" name="form_code" value="<?= $event_attributes['event_a']['form_code']; ?>">

                                                <input type="hidden" id="payment-event-code" class="form-control" name="amount" value="<?= $applicant_payment['fee_total']; ?>">
                                                <div class="cart_row">
                                                    <div class="col-50">
                                                        <div class="pull-right" style="margin-top:20px">
                                                            <button style="color: #2c2c2c;" type="submit" id="submit-apply" class="btn btn-success pull-left">
                                                                <i style="margin-right:4px; color:gold" class="fa fa-lg fa-lock" aria-hidden="true"></i> Submit Payment
                                                            </button>
                                                        </div>
                                                    </div>
                                                    <div class="col-50">
                                                        <?php if(!empty($applicant_payment['authnet_seal'])): ?>
                                                            <div class="pull-right" style="margin-top:0px; margin-right:0px;">
                                                                <?= $applicant_payment['authnet_seal'] ?>
                                                            </div>
                                                        <?php endif; ?>
                                                    </div>
                                                </div>
                                            </div>
                                    </form>
                                    </div>
                                </div>

                            <?php endif;
                            // END - generate paypal button and encrypted form
                            //////////////////////////////////
                            ?>

                                <?php echo $this->_display_payment_info($event_attributes['event_a']); ?>
                            </div><!-- mark1 -->
                            <?php
                         endif;
                    else:
                        ?>
                        <div style="margin-top: 10px;">
                            <?= $this->_get_approval_status($approval_status, $event_attributes['event_a']); ?>
                            <?= $this->_get_notice_by_pay_policy($event_attributes['event_a']['pay_policy']); ?>
                            <?= $this->_display_payment_info($event_attributes['event_a']); ?>
                        </div>
                        <?php
                    endif;
                endif;
            endif;
        endif;

        return ($form . $this->_generate_payment_screen_field(ob_get_clean(), $this->event_attributes));
    }

    /**
     * @param $formData
     * @param $form
     * @return string
     */
    private function _create_signature_screen_fields($formData, $form)
    {
        $attributes = json_decode($this->event_attributes, true);
        $return_url = $this->_get_skip_url(['amenities'=>'', 'pk'=>$_GET['pk']]);

        ob_start();

        ?>
        <div id="signature-pad" class="m-signature-pad">
            <div class="m-signature-pad--body">
                <canvas width="658" height="298" style="touch-action: none;">
            </div>
            <div class="m-signature-pad--footer">
                <div class="description">Use mouse or finger to sign above</div>
                <div class="left">
                    <button type="button" class="btn btn-success" data-action="clear">Clear</button>
                </div>
                <div class="right">
                    <button type="button" class="btn btn-success" data-action="save-svg"> Save Signature</button>
                </div>
            </div>
        </div>
        <script type="text/javascript">
            var wrapper = document.getElementById("signature-pad"),
                clearButton = wrapper.querySelector("[data-action=clear]"),
                savePNGButton = wrapper.querySelector("[data-action=save-png]"),
                saveSVGButton = wrapper.querySelector("[data-action=save-svg]"),
                canvas = wrapper.querySelector("canvas"),
                signaturePad;

            // Adjust canvas coordinate space taking into account pixel ratio,
            // to make it look crisp on mobile devices.
            // This also causes canvas to be cleared.
            function resizeCanvas() {
                // When zoomed out to less than 100%, for some very strange reason,
                // some browsers report devicePixelRatio as less than 1
                // and only part of the canvas is cleared then.
                var ratio =  Math.max(window.devicePixelRatio || 1, 1);
                canvas.width = canvas.offsetWidth * ratio;
                canvas.height = canvas.offsetHeight * ratio;
                canvas.getContext("2d").scale(ratio, ratio);
            }

            window.onresize = resizeCanvas;
            resizeCanvas();

            signaturePad = new SignaturePad(canvas);

            clearButton.addEventListener("click", function (event) {
                signaturePad.clear();
            });

            saveSVGButton.addEventListener("click", function (event) {
                if (signaturePad.isEmpty()) {
                    alert("Please provide signature first.");
                } else {
                    // window.open(signaturePad.toDataURL('image/svg+xml'));
                    var sig = {};
                    sig.svg_xml = signaturePad.toDataURL('image/svg+xml');
                    sig.svg = signaturePad.toDataURL('image/svg');
                    // console.log(sig);
                    jQuery.ajax({
                        url: "<?php echo admin_url('admin-ajax.php'); ?>",
                        type: "POST",
                        cache: false,
                        dataType: "json",
                        data: {svg_xml: sig.svg_xml, svg: sig.svg, dwc_screen_name: "signature", action: "dwc_signature_submission", dwc_verify_submission: "<?php echo wp_create_nonce("dwc_signature_submission");  ?>", form_code: "<?php echo $attributes['event_a']['form_code']; ?>", profile_key: "<?php echo $attributes['client_a']['profile_key']; ?>", terms_scope: "<?php echo $_GET['signature']; ?>"},
                        success: function(data){
                            if(data.status === "success"){
                                //alert('success');
                                location.href = "<?php echo $this->_get_skip_url(['terms' => '', 'pk' => $_GET['pk']]); ?>";
                            }else{
                                //alert('fail');
                                return false;
                            }
                        }
                    });
                }
            });

        </script>
        <?php
        return ob_get_clean();
    }

    /**
     * @param $formData
     * @param $form
     * @return string
     */
    private function _create_terms_screen_fields($formData, $form)
    {
        $terms_signed = $contract_signed = $no_signatures_required = $display_continue = $hide_continue = $mini_block = false;

        $event_attributes = json_decode($this->event_attributes, true);

        $status_approved = $event_attributes['applicant_a']['approval_status'] == 1 ? true : false;

        /*
         * $event_a['c_cfg2']
         * event_forms::c_cfg2
         * 1. e-sign before approval
         *      -require signature before initial payments
         * 2. e-sign after approval
         *      -do not require until approval status = 1
         */

        $contract_sign_period = $event_attributes['event_a']['c_cfg2'] == 1 ? 'pre-approval' : 'post-approval';

        if($event_attributes['event_a']['terms_id'] == null && $event_attributes['event_a']['contract'] == 0){
            $no_signatures_required = true;
        }else{
            $applicant_signature = json_decode($this->_get_data("signature", $event_attributes), true);
        }

        if(isset($applicant_signature['signature'])) {
            foreach($applicant_signature['signature'] AS $signature){
                if($signature['terms_scope'] == 1){
                    $terms_signed = true;
                    $terms_sign_date = $signature['sign_date'];
                    $terms_sign_svg = $signature['svg'];
                }
                if($signature['terms_scope'] == 2){
                    $contract_signed = true;
                    $contract_sign_date = $signature['sign_date'];
                    $contract_sign_svg = $signature['svg'];
                }
            }
        }

        if($event_attributes['event_a']['contract'] == 1 &&
            $contract_sign_period == 'post-approval' &&
            $status_approved == false) {
            $no_contract_signature_required = true;
        }

        // USE https://summernote.org/deep-dive/ FOR CONTRACT WINDOW

        ob_start();
        ?>

        <?php //// TERMS ///// ?>

        <?php if($formData['terms_scope'] == 1) : ?>

            <?php if($terms_signed) : ?>

                <div class="col-lg-offset-2">
                    <h4><?php echo $formData['terms']['title']; ?></h4>
                </div>

                <div id="summernote">
                    <?php echo $formData['terms']['body']; ?>
                </div>

                <div style="width: 100%; margin-bottom:20px;">
                    <div class="sig_block_inner">
                        <img src="<?php echo $terms_sign_svg; ?>" alt="" style="width: 80%;" height="auto">
                        <hr>
                        Signature
                    </div>
                    <div class="sig_block_inner">
                        <h4><?php echo $terms_sign_date; ?></h4>
                        <hr>
                        Date
                    </div>
                </div>
                <?php $display_continue = true; ?>
                <?php $mini_block = true; ?>

            <?php else: ?>

                <div class="col-lg-offset-2">
                    <h4><?php echo $formData['terms']['title']; ?></h4>
                </div>

                <div id="summernote">
                    <?php echo $formData['terms']['body']; ?>
                </div>

                <div style="width:80%; margin:20px auto; text-align:center;">
                    <form class="terms-checkbox">
                        <input id="terms-checkbox" type="checkbox"/><label class="fa" for="terms-checkbox"></label>
                        <?php echo $formData['terms']['note']; ?>
                    </form>
                </div>

                <div class="center" style="margin-top:10px;">
                    <button type="submit" id="terms-agree-button" class="btn">
                        <i class="fa fa-pencil" aria-hidden="true"></i> Sign Here
                    </button>
                </div>

                <script type="text/javascript">
                    jQuery(document).ready(function () {
                        // initialize jQuery stuff after page load
                        jQuery(function(){
                            jQuery('#terms-agree-button').click(function () {
                                document.location.href = "<?php echo $this->_get_skip_url(['terms'=>'', 'signature'=>$formData['terms_scope'], 'pk'=>$_GET['pk']]); ?>";
                            });
                            jQuery("#terms-checkbox").on("click", function() {
                                var chk = jQuery("#terms-checkbox").is(":checked");
                                jQuery('#terms-agree-button').prop("disabled", !chk).toggleClass("btn-success",chk);  // possibly add .button('refresh'); for JQM
                            });
                        });
                        jQuery('#terms-agree-button').prop("disabled", "disabled");
                    });
                </script>

                <?php $hide_continue = true; ?>
            <?php endif; ?>
        <?php endif; ?>

        <?php //// CONTRACTS ///// ?>

        <?php if($formData['terms_scope'] == 2) : ?>

            <?php if($contract_signed) : ?>

                <div class="col-lg-offset-2">
                    <h4><?php echo $formData['contract']['title']; ?></h4>
                </div>

                <div id="summernote">
                    <?php echo $formData['contract']['body']; ?>
                </div>

                <div style="width: 100%; margin-bottom:20px;">
                    <div class="sig_block_inner">
                        <img src="<?php echo $contract_sign_svg; ?>" alt="" style="width: 80%;" height="auto">
                        <hr>
                        Signature
                    </div>
                    <div class="sig_block_inner">
                        <h4><?php echo $contract_sign_date; ?></h4>
                        <hr>
                        Date
                    </div>
                </div>
                <?php $display_continue = true; ?>
                <?php $mini_block = true; ?>

            <?php else: ?>

                <div class="col-lg-offset-2">
                    <h4><?php echo $formData['contract']['title']; ?></h4>
                </div>

                <div id="summernote">
                    <?php echo $formData['contract']['body']; ?>
                </div>

                <div style="width:80%; margin:20px auto; text-align:center;">
                    <form class="terms-checkbox">
                        <input id="terms-checkbox" type="checkbox"/><label class="fa" for="terms-checkbox"></label>
                        <?php echo $formData['contract']['note']; ?>
                    </form>
                </div>

                <div class="center" style="margin-top:10px;">
                    <button type="submit" id="terms-agree-button" class="btn">
                        <i class="fa fa-pencil" aria-hidden="true"></i> Sign Here
                    </button>
                </div>

                <script type="text/javascript">
                    jQuery(document).ready(function () {
                        // initialize jQuery stuff after page load
                        jQuery(function(){
                            jQuery('#terms-agree-button').click(function () {
                                document.location.href = "<?php echo $this->_get_skip_url(['terms'=>'', 'signature'=>$formData['terms_scope'], 'pk'=>$_GET['pk']]); ?>";
                            });
                            jQuery("#terms-checkbox").on("click", function() {
                                var chk = jQuery("#terms-checkbox").is(":checked");
                                jQuery('#terms-agree-button').prop("disabled", !chk).toggleClass("btn-success",chk);  // possibly add .button('refresh'); for JQM
                            });
                        });
                        jQuery('#terms-agree-button').prop("disabled", "disabled");
                    });
                </script>

            <?php endif; ?>
        <?php endif; ?>

        <?php if($no_signatures_required): ?>

            <h4>No Signature Required.</h4>
            <?php $display_continue = true; ?>

        <?php elseif($no_contract_signature_required): ?>
            <div class="center" style="margin-top:10px;">
                <h5>Note: Final contract signature required AFTER application approval.</h5>
            </div>

            <?php $display_continue = $hide_continue ? false : true; ?>

        <?php endif; ?>

        <?php if($display_continue): ?>

            <div class="form-group">
                <div class="col-lg-offset-2 col-lg-11">
                    <button type="submit" id="skipTerms" class="btn btn-success pull-left">Continue <i style="margin-left:0px;" class="fa fa-chevron-right" aria-hidden="true"></i></button>
                </div>
            </div>
            <script type="text/javascript">
                jQuery(document).ready(function () {
                    // initialize jQuery stuff after page load
                    jQuery(function(){
                        jQuery('#skipTerms').click(function () {
                            document.location.href = "<?php echo $this->_get_skip_url(['payment'=>'', 'pk'=>$_GET['pk']]); ?>";
                        });
                    });
                });
            </script>

        <?php endif; ?>

        <?php if($mini_block): ?>
            <script type="text/javascript">
                var $ = jQuery.noConflict();
                jQuery(document).ready(function ($) {
                    $('#summernote').summernote({
                        toolbar: false,
                        height: 300
                    });
                });
            </script>
        <?php else: ?>
            <script type="text/javascript">
                var $ = jQuery.noConflict();
                jQuery(document).ready(function ($) {
                    $('#summernote').summernote({
                        toolbar: false,
                        height: 500
                    });
                });
            </script>
        <?php endif; ?>

        <?php
        return ($form . $this->_generate_terms_screen_field(ob_get_clean(), $this->event_attributes));
    }

    /**
     * @param $formData
     * @param $form
     * @return string
     */
    private function _create_map_screen_fields($formData, $form)
    {
        $attributes = json_decode($this->event_attributes, true);
        $show_border = $formData['maps_a']['show_border'] == 1 ? "true" : "false";
        $css_override_code = $formData['maps_a']['css_override'];

        $return_url = $this->_get_skip_url(['amenities'=>'', 'pk'=>$_GET['pk']]);
        ob_start();
        ?>
        <div id="container" class="container">
            <img id="mapster" src="<?php echo $formData['map_img_url']; ?>" alt="" usemap="#mapselector">
            <map id='mapselector' name='mapselector'>
                <?php
                if(isset($formData['imagemap_areas_a']) && count($formData['imagemap_areas_a'])):

                    foreach($formData['imagemap_areas_a'] as $key => $val):
                        echo $this->_generate_map_area($val);
                        ?>
                    <?php
                    endforeach;
                endif;
                ?>
            </map>
            <div class="topcorner"><a href="<?php echo $return_url; ?>" class="btn btn-success">Finish</a></div>
            <div id="note">
                Click on a booth area to select or de-select. <a id="close_note"><i class="fa fa-close" style="font-size:20px; color:red"></i></a>
            </div>
        </div>
        <script type="text/javascript">
            jQuery(document).ready(function () {
                // initialize jQuery stuff after page load
                jQuery(function(){
                    jQuery( '#mapster' ).mapster({
                        mapKey: 'key',
                        //staticState: true,
                        fill: true,
                        fillColor: '00ff00',
                        fillOpacity: 0.5,
                        stroke: <?php echo $show_border; ?>,
                        strokeWidth: 1,
                        strokeOpacity: 0.5,
                        strokeColor: 'fff',
                        showToolTip: true,
                        wrapClass: 'map_form',
                        toolTipContainer: "<div class='tooltip' style=''></div>",
                        areas: [<?php  echo implode(",",  $formData['area_status']);  ?>],
                        onClick: function (e) {
                            // alert(e.key); //201,202
                            var map_area_name = e.key;
                            jQuery.ajax({
                                url: "<?php echo admin_url('admin-ajax.php') ?>",
                                type: "POST",
                                cache: false,
                                data: {data:map_area_name, dwc_screen_name: "map", map: "<?php echo $_GET['map']; ?>", action: "dwc_map_submission", dwc_verify_submission: "<?php echo wp_create_nonce("dwc_map_submission");  ?>", form_code: "<?php echo $attributes['event_a']['form_code']; ?>", profile_key: "<?php echo $attributes['client_a']['profile_key']; ?>"},
                                success: function(data){
                                    // alert(status.toSource(data));
                                    /*
                                     * ['status' => 'unavailable'] - do nothing
                                     * ['status' => 'paid'] - do nothing
                                     * ['status' => 'removed' - done
                                     * ['status' => 'added' - done
                                     * ['status' => 'error-1'
                                     * ['status' => 'error-2']
                                     * */

                                    var status = jQuery.parseJSON(data);
                                    // alert(status.toSource());
                                    if(status['code'] == 'added'){
                                        var temp = status['detail'].split(" ");
                                        swal({
                                                title: temp[1] + " Added",
                                                text: "Detail: Booth " + temp[1] + ", Price $" + status['price'] + "\n(click booth again to de-select)",
                                                type: "success",
                                                showCancelButton: true,
                                                confirmButtonColor: "#DD6B55",
                                                confirmButtonText: "Yes, close.",
                                                cancelButtonText: "No, select another booth.",
                                                closeOnConfirm: true,
                                                closeOnCancel: true
                                            },
                                            function(isConfirm){
                                                if (isConfirm) {
                                                    window.location.href = "<?php echo $return_url; ?>";
                                                } else {
                                                    return false;
                                                }
                                            });

                                    }else if(status['code'] == 'removed'){
                                        var temp = status['detail'].split(" ");
                                        swal({
                                                title: temp[1] + " Removed",
                                                text: "Detail: Booth " + temp[1] + ", Price $" + status['price'],
                                                type: "info",
                                                showCancelButton: true,
                                                confirmButtonColor: "#DD6B55",
                                                confirmButtonText: "Yes, close.",
                                                cancelButtonText: "No, select another booth.",
                                                closeOnConfirm: true,
                                                closeOnCancel: true
                                            },
                                            function(isConfirm){
                                                if (isConfirm) {
                                                    window.location.href = "<?php echo $return_url; ?>";
                                                } else {
                                                    return false;
                                                }
                                            });
                                    }else if(status['code'] == 'paid'){
                                        // do nothing
                                    }else if(status['code'] == 'unavailable'){
                                        // do nothing
                                    }else{
                                        swal({
                                            title: "Error: " + status['code'],
                                            text: "Detail: Could not be added.",
                                            type: "error",
                                            showCancelButton: true,
                                            confirmButtonColor: "#DD6B55",
                                            confirmButtonText: "Yes, close.",
                                            cancelButtonText: "No, not done yet.",
                                            closeOnConfirm: true,
                                            closeOnCancel: true
                                        },
                                        function(isConfirm){
                                            if (isConfirm) {
                                                window.location.href = "<?php echo $return_url; ?>";
                                            } else {
                                                return false;
                                            }
                                        });
                                    }
                                }
                            });
                        }
                    });
                });
            });
        </script>
        <script>
            close = document.getElementById("close_note");
            close.addEventListener('click', function() {
                note = document.getElementById("note");
                note.style.display = 'none';
            }, false);
        </script>
        <style type="text/css">
            <?php echo $css_override_code; ?>
        </style>

        <?php
        return ob_get_clean();
    }

    /**
     * @param $formData
     * @param $form
     * @return string
     */
    private function _create_mapview_screen_fields($formData, $form)
    {
        global $wp;
        $current_url = home_url(add_query_arg([], $wp->request));

        $show_border = $formData['maps_a']['show_border'] == 1 ? "true" : "false";
        $css_override_code = $formData['maps_a']['css_override'];

        $applic_keywords_a = $formData['applic_keywords_a'];

        if(isset($formData['event_a']['home_url'])){
            $home_url = "<a href='{$formData['event_a']['home_url']}'>Home</a>";
        }else{
            $home_url = "<a href='/'>Home</a>";
        }

        if(isset($formData['event_a']['title'])) {
            $nav_header = "<div class='pull-left map-title-header'>{$home_url} > {$formData['event_a']['title']} > {$formData['maps_a']['name']}</div>";
        }

        if($formData['maps_a']['show_keywords'] == 1){

            $keyword_selector = '<div class="MapFlex-keywords">
                <select id="" class="keyword-select" name="keyword" onchange="if (this.value) window.location.href=this.value">
                    <option value="">Search by Keyword</option>';

            foreach($applic_keywords_a AS $kid => $keyword){
                $selected = $_GET['kid'] == $kid ? ' selected' : '';
                $keyword_selector .= "<option value='{$current_url}/?kid={$kid}'{$selected}>{$keyword}</option>";
            }
            $keyword_selector .= '</select>';
            $keyword_selector .= " <a href='{$current_url}' class='btn-keyword-reset' style='text-decoration: none; margin-bottom: 0px; margin-top: 0px;'>Reset</a>";
            $keyword_selector .= '</div>';
        }

        ob_start();
        ?>

        <header class="MapFlex-header">
            <?= $nav_header; ?>
            <?= $keyword_selector; ?>
        </header>

        <main class="MapFlex-body">
            <article class="MapFlex-content">
                <div class="bottom" id="map">
                    <img id="mapster" src="<?php echo $formData['map_img_url']; ?>" alt="" usemap="#mapselector">
                    <map id='mapselector' name='mapselector'>
                        <?php
                        if(isset($formData['imagemap_areas_a']) && count($formData['imagemap_areas_a'])):
                            foreach($formData['imagemap_areas_a'] as $key => $val):
                                echo $this->_generate_viewmap_area($val);
                            endforeach;
                        endif;
                        ?>
                    </map>
                </div>

                <script type="text/javascript">
                    jQuery(document).ready(function (e) {
                        // initialize jQuery stuff after page load

                        var img = jQuery('img');
                        var list = jQuery('#boothlist');

                        jQuery(function(e){
                            img.mapster({
                                mapKey: 'key',
                                fill: true,
                                stroke: 0,
                                scaleMap: true,
                                showToolTip: true,
                                toolTipContainer: "<div class='tooltip' style=''></div>",
                                areas: [<?php  echo implode(",",  $formData['area_status']);  ?>],
                                onClick: function (e) {
                                    // alert(e.key);
                                    if (e.state === 'highlight') {
                                        //relevantCheckbox = $(".mapsterCheckbox#" + e.key);
                                        //relevantCheckbox.prop("checked", !relevantCheckbox.prop("checked"));
                                        //relevantCheckbox = '';
                                    }
                                }
                            });
                        });

                        jQuery(document).on('mouseenter','#boothlist li',function(e) {
                            var el = jQuery(e.target);
                            // alert(el.attr('id'));

                            // tooltip
                            img.mapster('tooltip', el.attr('id'));
                            img.mapster('highlight',el.attr('id'));

                        }).on('mouseout','#boothlist li',function(e) {
                            e.preventDefault();
                            var el = jQuery(e.target);

                            // tooltip
                            img.mapster('tooltip');
                            img.mapster('highlight',false);
                        });

                        jQuery(function(e){
                            jQuery('#left').overlayScrollbars({
                                overflowBehavior : {
                                    x : "h",
                                    y : "scroll"
                                },
                                scrollbars : {
                                    visibility       : "auto",
                                    dragScrolling    : true,
                                    clickScrolling   : false,
                                    touchSupport     : true
                                },
                                textarea : {
                                    dynWidth  : true,
                                    dynHeight : false
                                }
                            });
                            jQuery('#right').overlayScrollbars({
                                overflowBehavior : {
                                    x : "h",
                                    y : "scroll"
                                },
                                scrollbars : {
                                    visibility       : "auto",
                                    dragScrolling    : true,
                                    clickScrolling   : false,
                                    touchSupport     : true
                                },
                                textarea : {
                                    dynWidth  : true,
                                    dynHeight : false
                                }
                            });
                        });
                    });
                </script>
            </article>

            <nav class="MapFlex-nav u-textCenter">
                <div id="owner" class="owner">
                    <ul id="boothlist" class="boothlist">
                        <?php
                        if(isset($formData['map_area_owner_a']) && count($formData['map_area_owner_a'])):
                            foreach($formData['map_area_owner_a'] as $applic_id => $client_name):
                                $x = 0;
                                do {
                                    ?>
                                    <li id='id_<?= $applic_id ?>'> [<?= $formData['applic_id_map_id_a'][$applic_id] ?>] <?= $client_name ?></li>
                                    <?php
                                    $x++;
                                } while ($x <= 0);
                            endforeach;
                        endif;
                        ?>
                    </ul>
                </div>

            </nav>
        </main>

        <style type="text/css">
            <?php echo $css_override_code; ?>
        </style>

        <?php
        return ob_get_clean();
    }

    /**
     * @param $formData
     * @param $form
     * @return string
     */
    private function _create_amenities_screen_fields($formData, $form)
    {
        global $wp;

        $event_attributes = json_decode($this->event_attributes, true);
        $uploads_attributes = json_decode($this->backup_attributes['uploads'], true);

        $current_url = home_url(add_query_arg([], $wp->request));
        $mt = str_replace('.', '', microtime(true));

        ob_start();
        ?>

        <?php
        // mod here to change dialogue if no amenities
        if(count($formData['amenities_selector_a'])){
        ?>

        <div class="col-lg-offset-2">
            <?php if(!empty($formData['amenity_note'])): ?>
                <div style="margin-left:20px; margin-top:4px;"><?= $formData['amenity_note'] ?></div>
            <?php else: ?>
                <h4>Select the amenities you require.</h4>
            <?php endif; ?>
        </div>

        <?php }else{ ?>

            <div class="col-lg-offset-2">
                <br><h4>No amenities to select for this event.</h4>
            </div>

        <?php } ?>

        <div id="amenities_container">

        <?php
        $map_select_bool = '';
        $map_selection_required = $event_attributes['event_a']['booth_required'] == 1 ? "*" : "";
        $map_policy = $event_attributes['event_a']['map_policy'];

        $map_reservation_shared_a = $formData['map_reservation_shared_a'];
        $map_reservation_share_with_name_a = $formData['map_reservation_share_with_name_a'];
        $map_reservation_fee_override_a = $formData['map_reservation_fee_override_a'];

        if(count($formData['amenities_selector_a'])) {

            foreach ($formData['amenity_types_custom_order_a'] as $key => $val) { ?>

                <?php if (array_key_exists($val, $formData['amenity_types_map_selectors_a'])) { ?>

                    <div class="form-group field-event required">
                        <label class="col-lg-2 control-label"
                               for="amenity-types"><?php echo $formData['amenity_types_a'][$val].$map_selection_required; ?>
                        </label>

                        <div class="col-lg-2 control-desc"
                             for="amenity-types"><?php echo $formData['amenity_types_desc_a'][$val]; ?></div>

                        <?php foreach ($formData['amenities_selector_a'][$val] as $selectorKey => $selector): ?>

                            <?php $mapData = $this->_get_map_data_by_id($selectorKey, $_GET['pk']); ?>

                            <div class="col-lg-10">
                                <input type="hidden" name="amenity_types[amenity_types_id]">

                                <div id="clients-form">

                                    <?php if($map_policy == 2): ?>
                                        <a href="<?php echo $this->_get_skip_url(['amenities' => '', 'map' => $selectorKey, 'pk' => $_GET['pk']]); ?>"
                                            class="btn btn-success" style="text-decoration: none; margin-bottom: 6px; margin-top: 6px;">Select <?php echo $selector; ?></a>
                                    <?php endif; ?>

                                    <?php if (count($formData['amenity_map_area_selected_a'])): ?>
                                        <?php foreach ($formData['amenity_map_area_selected_a'] as $areaKey => $area): ?>
                                            <?php if ($area == $selectorKey): ?>
                                                <?php
                                                $map_select_bool = 'true';
                                                $area_data = $this->_get_area_data_by_id($areaKey, $mapData);
                                                list(, $href) = explode(".", $area_data['href'], 2);

                                                if(isset($map_reservation_fee_override_a[$area_data['map_area_id']])){
                                                    $area_data['alt'] = $map_reservation_fee_override_a[$area_data['map_area_id']];
                                                }
                                                ?>
                                                <div style="margin-bottom: 0px; display: flex; justify-content: flex-start;">
                                                    <div id="load_map_<?php echo $val; ?>"
                                                         class="amenity-label checkbox">

                                                        <input id="" class="amenity_type" type="checkbox" value="1" checked
                                                            <?php if($map_policy == 2): ?>
                                                               onclick="toMap('<?php echo $this->_get_skip_url(['amenities' => '', 'map' => $selectorKey, 'pk' => $_GET['pk']]) ?>')"
                                                            <?php else: ?>
                                                               onclick="this.checked=true;"
                                                            <?php endif; ?>
                                                        >

                                                        <label for=""><?php echo $href; ?>
                                                            <?php
                                                            if(!empty($area_data['title'])): echo " (".$area_data['title'] .")"; endif;
                                                            ?>
                                                            </label>
                                                        <?php
                                                        if(!empty($area_data['reserved']) && !isset($formData['amenity_map_area_paid_a'][$area_data['map_area_id']])):
                                                            $current_url .= "/?amenities&maprel=".$area_data['map_area_id']."&pk=" . $_GET['pk'] . "&v=" . $mt;
                                                            echo '<a href="'.$current_url.'" class="maprel btn-mini btn-red" data-title="Release reserved location?">release</a>';
                                                        endif;
                                                        ?>

                                                        <?php
                                                        if(!empty($area_data['reserved']) && isset($formData['map_reservations_expiration_a'][$area_data['map_area_id']]) && !isset($formData['amenity_map_area_paid_a'][$area_data['map_area_id']])):
                                                            $expires_date = date("m/d/Y", strtotime($formData['map_reservations_expiration_a'][$area_data['map_area_id']]));
                                                            ?>
                                                            <div class="reserved_date tooltip" title="Reservations must be paid by the hold date or it will be released">
                                                                Booth reserved through <?= $expires_date ?>
                                                                <i class="fa fa-info-circle" aria-hidden="true"></i></div>
                                                        <?php
                                                        endif;
                                                        ?>

                                                        <?php
                                                        if(isset($map_reservation_shared_a[$area_data['map_area_id']])):
                                                            $shared_with = isset($map_reservation_share_with_name_a[$area_data['map_area_id']]) ? $map_reservation_share_with_name_a[$area_data['map_area_id']] : '';
                                                            ?>
                                                            <div class="shared_with tooltip" title="This booth is shared by you and another party">
                                                                Shared with: <?= $shared_with ?></div>
                                                        <?php
                                                        endif;
                                                        ?>

                                                        <?php
                                                        if($formData['booth_sharing'] == 1):
                                                            ?>
                                                            <span style="font-size: 12px; float: right; margin: 0; line-height: 8px; cursor: pointer" id="booth_share_<?php echo $val; ?>">share space</span>
                                                            <div style="display: none" id="booth_share_form_<?= $val; ?>">
                                                                e-mail of the second party
                                                                <input id="booth_share_email_<?= $val; ?>" class="form-control" name="Clients[booth_share_email][<?= $val; ?>]" maxlength="255" value="" style="max-width: 300px" type="text">
                                                                What percentage of the cost are THEY paying?
                                                                <input id="booth_share_email_<?= $val; ?>" class="form-control" name="Clients[booth_share_percent][<?= $val; ?>]" maxlength="255" value="" style="max-width: 300px" type="text">
                                                            </div>
                                                            <?php
                                                        endif;
                                                        ?>
                                                    </div>
                                                    <div class="col-lg-8 amenity-price">
                                                        <?php echo "\$".$area_data['alt']; ?>
                                                        <strong>x</strong>
                                                        <input type="text" class="amenity_booth_qty" size="2" maxlength="2"
                                                               name="" value="1" disabled>
                                                    </div>
                                                    <div class="flex-right-gutter"></div>
                                                </div>

                                                <script type="text/javascript">
                                                    jQuery(document).ready(function () {
                                                        // initialize jQuery stuff after page load
                                                        new jBox('Modal', {
                                                            attach: '#booth_share_<?= $val; ?>',
                                                            animation: 'zoomIn',
                                                            overlay: false,
                                                            title: 'Share <?= $href; ?> with a second party',
                                                            content: jQuery('#booth_share_form_<?= $val; ?>')
                                                        });
                                                    });
                                                </script>

                                            <?php endif; ?>

                                        <?php endforeach; ?>
                                    <?php endif; ?>
                                </div>
                            </div>
                        <?php endforeach; ?>
                        <?php
                        if($formData['booth_required'] == 1):
                            echo "<input data-parsley-error-message='A booth selection is required. Please click on a map button and select a booth.' style='display:none;' required='' name='booth_selections' type='text' value='{$map_select_bool}'>";
                        endif;
                        ?>

                    </div>

                <?php } else {

                    $selector_type = $formData['amenity_types_selector_a'][$val] == 1 ? 'checkbox' : 'radio';
                    ?>

                    <?php if (array_key_exists($val, $formData['amenities_selector_a'])) { ?>

                        <div class="form-group field-event required">
                            <label class="col-lg-2 control-label"
                                   for="amenity-types"><?php echo $formData['amenity_types_a'][$val]; ?></label>

                            <div class="col-lg-2 control-desc"
                                   for="amenity-types"><?php echo $formData['amenity_types_desc_a'][$val]; ?></div>

                            <div class="col-lg-10" style="">
                                <input type="hidden" name="amenity_types[amenity_types_id]" value="">

                                <div id="clients-form">
                                    <?php foreach ($formData['amenities_selector_a'][$val] as $selectorKey => $selector): ?>
                                        <div style="display: flex; justify-content: flex-start;">
                                            <div class="amenity-label <?= $selector_type; ?>">
                                                <input type="<?= $selector_type; ?>"
                                                       <?php  // onclick="amenitySelect()" ?>
                                                       id="amenity_type_<?php echo $selectorKey; ?>"
                                                       class="amenity_type" <?php echo (isset($formData['amenity_id_selected_a'][$val]) && in_array($selectorKey, $formData['amenity_id_selected_a'][$val])) ? "checked" : ""; ?>
                                                       name="Clients[amenity_types][<?php echo $val; ?>][]"
                                                       value="<?php echo $selectorKey; ?>">

                                                <label for="amenity_type_<?php echo $selectorKey; ?>" class="tooltip" title="<?= $formData['amenities_selector_desc_a'][$val][$selectorKey] ?>">
                                                    <?php echo $selector; ?>
                                                    <i class="fa fa-info-circle" aria-hidden="true"></i>
                                                </label>
                                            </div>
                                            <div class="col-lg-8 amenity-price">
                                                <?php echo "\$".$formData['amenities_base_rate_a'][$selectorKey]; ?>
                                                <span><strong>x</strong></span>

                                                <?php
                                                // multi_qty 1 = show qty selector, 0 = default to 1
                                                if($formData['amenities_multi_qty_a'][$selectorKey] == 1):
                                                ?>
                                                    <input id="amenity_qty_<?php echo $selectorKey; ?>"
                                                           class="amenity_qty"
                                                           size="2"
                                                           maxlength="2"
                                                           name="amenity_qty[<?php echo $selectorKey; ?>]"
                                                           type="number"
                                                           value="<?php echo (isset($formData['amenity_qty_selected_a'][$selectorKey])) ? $formData['amenity_qty_selected_a'][$selectorKey] : 0; ?>">

                                                <?php else: ?>
                                                    <input id="amenity_qty_<?php echo $selectorKey; ?>"
                                                           class="amenity_qty_single"
                                                           size="2"
                                                           maxlength="2"
                                                           name="amenity_qty[<?php echo $selectorKey; ?>]"
                                                           type="number"
                                                           value="<?php echo (isset($formData['amenity_qty_selected_a'][$selectorKey])) ? $formData['amenity_qty_selected_a'][$selectorKey] : 0; ?>">
                                                <?php endif; ?>
                                            </div>
                                            <div class="flex-right-gutter"></div>
                                        </div>
                                    <?php endforeach; ?>
                                </div>
                            </div>
                        </div>
                        <div style="margin-bottom: 15px;"></div>
                    <?php
                    }
                }
            }
        }
        ?>

        <?php // endforeach; ?>

            <div class="form-group field-clients-value-client_profile_key">
                <div class="col-lg-8">
                    <input type="hidden" id="clients-value-client_profile_key" class="form-control" name="Clients[Value][client_profile_key]" value="<?php echo $event_attributes['client_a']['profile_key']; ?>">
                </div>
            </div>
            <div class="form-group field-clients-value-applic_profile_key">
                <div class="col-lg-8">
                    <input type="hidden" id="clients-value-applic_profile_key" class="form-control" name="Clients[Value][applic_profile_key]" value="<?php echo $event_attributes['applicant_a']['profile_key']; ?>">
                </div>
            </div>
            <div class="form-group field-clients-value-apply_step3">
                <div class="col-lg-8">
                    <input type="hidden" id="clients-value-apply_step3" class="form-control" name="Clients[Value][apply_step3]" value="1">
                </div>
            </div>
            <div class="form-group">
                <div class="col-lg-8">
                    <input type="hidden" id="clients-value-event-code" class="form-control" name="Clients[Value][form_code]" value="<?php echo $event_attributes['event_a']['form_code']; ?>">
                </div>
            </div>
            <div class="col-lg-offset-2 col-lg-11">
            <?php
                if(count($formData['amenities_selector_a'])) {
                    echo '<button type="submit" id="submit-apply" class="btn btn-success pull-left">Save & Continue <i style="margin-left:0px;" class="fa fa-chevron-right" aria-hidden="true"></i></button>';
                }else{
                    echo '<button type="submit" id="submit-apply" class="btn btn-success pull-left">Next</button>';
                }
            ?>
                <div id="submit-spinner" class="pull-left"></div>
            </div>
        </div>
        <?php

        return ($form . $this->_generate_amenities_screen_field(ob_get_clean(), $this->event_attributes));
    }

    /**
     * @param $formData
     * @param $form
     * @return string
     */
    private function _create_extra_screen_fields($formData, $form)
    {
        global $wp;

        $event_attributes = json_decode($this->event_attributes, true);
        $uploads_attributes = json_decode($this->backup_attributes['uploads'], true);

        $current_url = home_url(add_query_arg([], $wp->request));
        $mt = str_replace('.', '', microtime(true));

        ob_start();

            // print_r($formData); die;

            foreach($formData['am_field_amenity_id_a'] as $amenity_id => $val){
                if(isset($formData['amenity_qty_selected_a'][$amenity_id])) {
                    $amenity_field_id_selected_qty_a[$amenity_id] = $formData['amenity_qty_selected_a'][$amenity_id];
                }
            }

            foreach ($amenity_field_id_selected_qty_a AS $amenity_id => $extra_qty) {
                $c = 0;
                ?>
                <div style="margin-bottom:10px;">
                    <h4><?= $formData['amenities_label_a'][$amenity_id] ?></h4>
                    <div style="margin-top:4px;"><?= $formData['amenities_desc_a'][$amenity_id] ?></div>
                </div>

                <div id="amenities_container">

                <?php
                $amenity_field_id_sorted_a = [];
                foreach ($formData['am_field_amenity_id_a'] AS $amenity_field_id => $amenity_field_id_a) {
                    foreach($amenity_field_id_a AS $amenity_field_id => $tmp){
                        if(trim($formData['am_field_order_a'][$amenity_id][$amenity_field_id]) !== '') {
                            $amenity_field_id_sorted_a[$formData['am_field_order_a'][$amenity_id][$amenity_field_id]] = $amenity_field_id;
                        }
                    }
                }

                ksort($amenity_field_id_sorted_a);

                while($extra_qty > $c) {
                    $c++;

                    echo '<div class="amenity-title" style=""><h4>'.$formData['amenities_label_a'][$amenity_id].' #'.$c.'</h4></div>';
                    echo '<div class="badge_block">';

                    foreach($amenity_field_id_sorted_a AS $amenity_field_id){

                        $max_width = '';
                        $required = 'required';
                        $filter_type = '';

                        $value = isset($formData['am_field_data_values_a'][$amenity_id][$c]) ? $formData['am_field_data_values_a'][$amenity_id][$c][$amenity_field_id] : '';
                        ?>

                            <div class="form-group field-clients-name">
                                <label class="col-lg-2 control-label" for="extra-<?= $amenity_field_id; ?>-<?= $c; ?>"><?= $formData['am_field_labels_a'][$amenity_id][$amenity_field_id]; ?></label>
                                <div class="col-lg-8">
                                    <input type="text" id="extra-<?= $amenity_field_id; ?>-<?= $c; ?>" class="form-control" name="amenity_extra[amenity_field_id_<?= $amenity_id ?>-<?= $amenity_field_id; ?>-<?= $c; ?>][<?= $amenity_id ?>][<?= $amenity_field_id; ?>]" maxlength="255" value="<?= $value; ?>" style="<?= $max_width; ?>" <?= $filter_type; ?> <?= $required; ?>>
                                </div>
                            </div>

                        <?php
                    }
                    echo '</div>';
                }
                echo '</div><!-- END amenities_container -->';
            }
            ?>

            <?php // endforeach; ?>
            <div id="amenities_container">

            <div class="form-group field-clients-value-client_profile_key">
                <div class="col-lg-8">
                    <input type="hidden" id="clients-value-client_profile_key" class="form-control" name="Clients[Value][client_profile_key]" value="<?php echo $event_attributes['client_a']['profile_key']; ?>">
                </div>
            </div>
            <div class="form-group field-clients-value-applic_profile_key">
                <div class="col-lg-8">
                    <input type="hidden" id="clients-value-applic_profile_key" class="form-control" name="Clients[Value][applic_profile_key]" value="<?php echo $event_attributes['applicant_a']['profile_key']; ?>">
                </div>
            </div>
            <div class="form-group field-clients-value-apply_step3">
                <div class="col-lg-8">
                    <input type="hidden" id="clients-value-apply_step3" class="form-control" name="Clients[Value][apply_step3]" value="1">
                </div>
            </div>
            <div class="form-group">
                <div class="col-lg-8">
                    <input type="hidden" id="clients-value-event-code" class="form-control" name="Clients[Value][form_code]" value="<?php echo $event_attributes['event_a']['form_code']; ?>">
                </div>
            </div>
            <div class="col-lg-offset-2 col-lg-11">
                <?php
                if(count($formData['amenities_selector_a'])) {
                    echo '<button type="submit" id="submit-apply" class="btn btn-success pull-left">Save & Continue <i style="margin-left:0px;" class="fa fa-chevron-right" aria-hidden="true"></i></button>';
                }else{
                    echo '<button type="submit" id="submit-apply" class="btn btn-success pull-left">Next</button>';
                }
                ?>
                <div id="submit-spinner" class="pull-left"></div>
            </div>
        </div>
        <?php

        return ($form . $this->_generate_amenities_screen_field(ob_get_clean(), $this->event_attributes));
    }


    /**
     * @param $formData
     * @param $form
     * @return string
     */
    private function _create_uploads_screen_fields($formData, $form)
    {
        $event_attributes = json_decode($this->event_attributes, true);
        ob_start();

        if(count($formData['missed_docs_exist_a']) > 0){
            echo '<div class="red-alert">';
            echo count($formData['missed_docs_exist_a']) == 1 ? "1 required file missing." : count($formData['missed_docs_exist_a'])." required files missing.";
            echo '</div>';
        }

        $formData['lic_id_field_label_a'] = isset($formData['lic_id_field_label_a']) ? $formData['lic_id_field_label_a'] : [];

        foreach($formData['lic_id_field_label_a'] as $val):
            $current_field_key = explode("_", $val);
            $current_field_key = end($current_field_key);

            $event_doc_id = isset($formData['client_lic_event_doc_id_a'][$current_field_key]) ? $formData['client_lic_event_doc_id_a'][$current_field_key] : '';

            $doc_filename = isset($formData['event_docs_filename_a'][$event_doc_id]) ? $formData['event_docs_filename_a'][$event_doc_id] : false;
            $doc_key_url = isset($formData['event_doc_key_a'][$event_doc_id]) ? $formData['event_doc_key_a'][$event_doc_id] : false;

            $allowedFileExtensions = $formData['client_lic_doctype_a'][$current_field_key] == 1 ? ['doc', 'pdf', 'docx'] : ['png', 'gif', 'jpg', 'jpeg'];
        ?>

            <div class="form-group field-dynamicmodel-<?php echo $val; ?> widget-box file-upload-container">
                <div class="widget-header">
                    <?php
                    if($doc_filename) {
                        ?>
                        <button title="Download <?= $doc_filename; ?>" type="button" id="doc_download_<?= $doc_key_url; ?>" class="tooltip btn btn-form-download" href="<?= $this->base_site_url; ?>/events/download?dk=<?= $doc_key_url; ?>">
                            <i class="fa fa-arrow-down" aria-hidden="true"></i> Download
                        </button>
                        <?php
                    }
                    ?>
                    <div class="widget-title">
                        <h3><?php echo $formData['client_lic_label_a'][$current_field_key]; ?></h3>
                    </div>
                    <div class="widget-subtitle">
                        <?php echo $formData['client_lic_desc_a'][$current_field_key]; ?>
                    </div>
                </div>

                <div class="widget-body">
                    <div class="widget-main">

                        <?php if(isset($formData['client_lic_thumb_a'][$current_field_key])) : ?>
                            <div class="doc-thumb">
                                <img src="<?= $formData['client_lic_thumb_a'][$current_field_key] ?>">
                            </div>
                        <?php endif; ?>


                            <input type="hidden" name="DynamicModel[<?php echo $val; ?>]">
                            <input type="file" name="DynamicModel[<?php echo $val; ?>]" id="dynamicmodel-<?php echo $val; ?>" class="file-loading">

                        <div>
                            <p class="help-block">
                                <strong>File types</strong>:
                                <?php
                                echo implode(", ",$allowedFileExtensions);
                                ?>.
                                <strong>Size Limit:</strong> <?= $this->_get_php_limits()['upload_max_filesize']; ?></p>
                            <div class="doc-status">
                                <?php if(!in_array($current_field_key, $formData['lic_id_missing_a'])): ?>
                                    <?php echo $this->_uploaded_file_status($current_field_key, $formData); ?>
                                <?php else: ?>
                                    <span style="font-size: 17px;" class="red-alert">
                                            <i class="fa fa-times"></i>
                                        </span>
                                    Not uploaded yet.
                                    <?php if(in_array($current_field_key,$formData['lic_id_required_a'])): ?>
                                        <span class="red-alert"><strong>(Required)</strong></span>
                                    <?php endif; ?>
                                <?php endif; ?>
                            </div>
                        </div>

                        <?php if($event_attributes['event_a']['rec_type'] == 2): ?>
                            <table style="width: 100%;" class="table table-striped table-bordered detail-view">
                                <tbody>
                                <tr>
                                    <th style="white-space: nowrap; width:80px;">Title</th>
                                    <td style="text-align: left;">
                                        <input type="text" name="DynamicModel[meta][lic_title_<?= $current_field_key; ?>]" id="dynamicmodel-lic_title_<?= $current_field_key; ?>" class="form-control" maxlength="100" value="<?= $formData['client_docs_meta_a'][$current_field_key]['title']; ?>">
                                    </td>
                                </tr>
                                <tr>
                                    <th style="white-space: nowrap;">Description</th>
                                    <td style="text-align: left;">
                                        <textarea name="DynamicModel[meta][lic_desc_<?= $current_field_key; ?>]" id="dynamicmodel-lic_desc_<?= $current_field_key; ?>" class="form-control"><?= $formData['client_docs_meta_a'][$current_field_key]['description']; ?></textarea>
                                    </td>
                                </tr>
                                <tr>
                                    <th style="white-space: nowrap;">Dimensions</th>
                                    <td style="text-align: left;">
                                        <input type="text" name="DynamicModel[meta][lic_dims_<?= $current_field_key; ?>]" id="dynamicmodel-lic_dims_<?= $current_field_key; ?>" class="form-control" maxlength="50" value="<?= $formData['client_docs_meta_a'][$current_field_key]['dims']; ?>">
                                    </td>
                                </tr>
                                <tr>
                                    <th style="white-space: nowrap;">Price</th>
                                    <td style="text-align: left;">
                                        <div class="input-group">
                                            <span class="input-group-addon">
                                                <i class="fa fa-usd" aria-hidden="true"></i>
                                            </span>
                                            <input class="form-control" style="width:100px;" type="text" name="DynamicModel[meta][lic_price_<?= $current_field_key; ?>]" id="dynamicmodel-lic_price_<?= $current_field_key; ?>" class="form-control" maxlength="10" value="<?= $formData['client_docs_meta_a'][$current_field_key]['price']; ?>">
                                        </div>
                                    </td>
                                </tr>

                                </tbody>
                            </table>

                        <?php endif; ?>


                    </div><!-- /.widget-main -->
                </div><!-- /.widget-body -->
            </div><!-- /.widget-box -->

        <?php
        //
            endforeach;
            ?>
            <div class="form-group">
                <div class="col-lg-8">
                    <input type="hidden" name="form_code"
                           value="<?php echo isset($event_attributes['event_a']['form_code']) ? $event_attributes['event_a']['form_code'] : ''; ?>">
                </div>
            </div>
            <div class="form-group field-dynamicmodel-value-client_profile_key">
                <div class="col-lg-8">
                    <input type="hidden" id="dynamicmodel-value-client_profile_key" class="form-control"
                           name="DynamicModel[Value][client_profile_key]"
                           value="<?php echo isset($event_attributes['client_a']['profile_key']) ? $event_attributes['client_a']['profile_key'] : ''; ?>">
                </div>
            </div>
            <div class="form-group field-dynamicmodel-value-applic_profile_key">
                <div class="col-lg-8">
                    <input type="hidden" id="dynamicmodel-value-applic_profile_key" class="form-control"
                           name="DynamicModel[Value][applic_profile_key]"
                           value="<?php echo isset($event_attributes['applicant_a']['profile_key']) ? $event_attributes['applicant_a']['profile_key'] : ''; ?>">
                </div>
            </div>
            <div class="form-group field-dynamicmodel-value-apply-step2">
                <div class="col-lg-8">
                    <input type="hidden" id="dynamicmodel-value-apply_step2" class="form-control"
                           name="DynamicModel[Value][apply_step2]" value="1">
                </div>
            </div>

            <?php

        if(count($formData['lic_id_field_label_a']) == 0){
            ?>
            <h4>No Uploads Required.</h4>
            <div class="form-group">
                <div class="col-lg-offset-2 col-lg-11">
                    <button type="submit" class="btn btn-success pull-left">Continue <i style="margin-left:0px;" class="fa fa-chevron-right" aria-hidden="true"></i></button>
                    <div class="hidden">
                        <a href="<?php echo $this->_get_skip_url(["amenities" => '', "pk" => $_GET['pk']]); ?>"
                           id="skipFilesUpload"></a>
                    </div>
                </div>
            </div>

            <?php
        }else{
            echo '<div class="well well-sm" style="margin-bottom:10px; margin-top:10px;">
                <i style="float:left; margin-right:10px;" class="fa fa-2x fa-exclamation-circle blue" aria-hidden="true"></i>
                Before uploading, please make sure your files are <strong>no larger than '.str_replace('M',' Megs',$this->_get_php_limits()["upload_max_filesize"]).'
                each.</strong>. Also see: <a href="https://www.google.com/search?num=10&q=how+to+resize+image+files" target="_blank">How to resize images</a></div>';
            ?>

            <div class="form-group">
                <div class="col-lg-offset-2 col-lg-11">
                    <button type="submit" class="btn btn-success pull-left">Upload Docs</button>
                    <button type="submit" class="btn btn-info pull-left" style="margin-left: 6px;"
                            onclick="skipFilesUpload()">Skip
                    </button>
                    <div id="submit-uploading" class="pull-left"></div>
                    <div id="submit-spinner" class="pull-left"></div>
                    <div class="hidden">
                        <a href="<?php echo $this->_get_skip_url(["amenities" => '', "pk" => $_GET['pk']]); ?>"
                           id="skipFilesUpload"></a>
                    </div>
                </div>
            </div>

            <?php
        }

        return ($form . $this->_generate_uploads_screen_field(ob_get_clean(), $this->event_attributes));
    }

    /**
     * @param $formData
     * @param $form
     * @return string
     */
    private function _create_error_screen($formData, $form)
    {
        ob_start();
        ?>

        <div class="form-group">
            <div class="col-lg-offset-2 col-lg-11">
                <div style="text-align: center;">
                    <?php if($_GET['status'] == '410' || $_GET['status'] == '411'): ?>
                        <img style="width: 200px; height: 183px;"
                             src="<?php echo REGISTERONE_WORDPRESS_CLIENT_URL; ?>/front-end/assets/img/magnifying-glass.png"
                             alt="broken">
                            <!-- see: https://pixabay.com/en/magnifying-glass-lense-loupe-97635/ -->
                    <?php else: ?>
                        <img style="width: 192px; height: 300px;"
                             src="<?php echo REGISTERONE_WORDPRESS_CLIENT_URL; ?>/front-end/assets/img/broken-roger.png"
                             alt="broken">
                    <?php endif; ?>
                    <br>
                    <?php
                    if($_GET['status'] == '409') {
                        echo "<h3><span class='red-alert'>Your email has already been used for this event application.</span></h3><br>
                        <strong>I just sent you a message with your existing application profile link.</strong><br>
                        <strong>Please check your email or phone now for the application profile link, or
                        <a href='https://app.registerone.com/signup' target='_blank'>create a vendor account</a> on RegisterONE.</strong><br>";

                        echo "Status Code: {$_GET['status']}";

                    }elseif($_GET['status'] == '410') {
                        echo "<h2><span class='green-alert'>Your application profile has been found!</span></h2>
                        <strong>I just sent you a message with your existing application link.</strong><br>
                        <strong>Please check your email or phone now, or
                        <a href='https://app.registerone.com/signup' target='_blank'>create a vendor account</a> on RegisterONE.</strong><br>";

                        echo "Status Code: {$_GET['status']}";

                    }elseif($_GET['status'] == '411') {
                        echo "<h2><span class='red-alert'>Your application profile was not found with this email!</span></h2>
                        <strong>If you did fill out an application already, maybe you registered with a different email address.</strong><br>
                        <strong>Please check your email for the original application profile link, or
                        <a href='https://app.registerone.com/signup' target='_blank'>create a vendor account</a> on RegisterONE.</strong><br>";

                        echo "Status Code: {$_GET['status']}";

                    }elseif($formData['code'] == 0 && $formData['status'] == 404) {
                        echo "I couldn't find this application profile.<br>Please check the URL and make sure it's complete and not missing anything!<br>";
                        echo "Status Code: {$formData['status']}";

                    }elseif($formData['code'] == 0 && $formData['status'] == 503) {
                        echo "Application is currently down for maintenance.<br>Please try again at a later time and thank you for your patience.<br>";
                        echo "Status Code: {$formData['status']}";

                    }elseif($formData['code'] == 0 && $formData['status'] == 402) {
                        echo "<strong>Pre-Launch applicant limit reached!</strong><br>
                        This event is in Pre-Launch and has reached it's maximum applicants.<br>
                        To accept more vendors, please launch the event from RegisterONE>Events or contact the event administrator.<br>";

                        echo "Status Code: {$formData['status']}";

                    }else{
                        echo "Sorry, an error has occurred.<br>";
                    }

                    ?>
                    <br><br>

                    <button onclick="goBack()" class="btn btn-error">Go Back</button>

                    <script>
                        function goBack() {
                            window.history.go(-1);
                        }
                    </script>
                </div>
            </div>
        </div>

        <?php

        return $form . ob_get_clean();
    }

    /**
     * @param $formData
     * @param $form
     * @return string
     */
    private function _create_profile_screen_fields($formData, $form)
    {
        $client_a = isset($formData['client_a']) ? $formData['client_a'] : [];
        $event_a = $formData['event_a'];
        $pre_launch_count = $formData['pre_launch_count'];

        // field types (from profile_fields::field_type) that are stored in json_encoded array (in applicant_field_data::text_long)
        $selector_fields = ['dropdown','radio','checkboxes'];

        ob_start();

        $input_data = '';
        if(isset($formData['profile_field_data_a'])) {
            foreach ($formData['profile_field_data_a'] AS $input_data) {
                if (isset($input_data['field_type'])) {

                    if (in_array($input_data['field_type'], $selector_fields)) {
                        $input_data['option_values_a'] = json_decode($input_data['option_values_a'], true);
                    }

                    if($input_data['model_field'] == 'client_types_a' && $event_a['client_type_hide'] == 1){
                        $hidden_input_client_type = '<input type="hidden" name="Clients[client_applic_type][]" value="'.$event_a['client_types'][0].'">';
                        continue;
                    }

                    echo $this->_generate_input_field($input_data['field_type'], $input_data);
                }
            }
        }
        ?>

        <div class="form-group field-clients-value-apply_step1">
            <div class="col-lg-8">
                <input type="hidden" id="clients-value-apply_step1" class="form-control" name="Clients[Value][apply_step1]" value="1">
            </div>
        </div>

        <div class="form-group">
            <div class="col-lg-offset-2 col-lg-11">
                <button type="submit" id="submit-apply" class="btn btn-success pull-left">
                <?php if(isset($_GET['pk']) && $_GET['pk'] != ''){ ?>
                    Save & Continue <i style="margin-left:0px;" class="fa fa-chevron-right" aria-hidden="true"></i>
                <?php }elseif(isset($_GET['return'])){ ?>
                    Find Application Profile
                <?php }else{ ?>
                    Next
                <?php } ?>
                </button>
                <div id="submit-spinner" class="pull-left"></div>
            </div>
        </div>

        <?php if(isset($hidden_input_client_type)): ?>
            <?= $hidden_input_client_type; ?>
        <?php endif; ?>

        <?php if(isset($_GET['return']) && $_GET['return'] != ''): ?>
            <input type="hidden" name="return_vendor" value="1">
        <?php endif; ?>

        <?php if(isset($client_a['profile_key']) && $client_a['profile_key'] != ''): ?>
            <input type="hidden" name="profile_key" value="<?php echo $client_a['profile_key']?>">
        <?php endif; ?>

        <?php if(isset($event_a['form_code']) && $event_a['form_code'] != ''): ?>
            <input type="hidden" name="form_code" value="<?php echo $event_a['form_code']?>">
        <?php endif; ?>

        <?php if(isset($_GET['rfc']) && $_GET['rfc'] != ''): ?>
            <input type="hidden" name="rfc" value="<?php echo $_GET['rfc']?>">
        <?php endif;

        return ($form . $this->_generate_profile_screen_field(ob_get_clean(), $client_a, $event_a, $pre_launch_count));
    }

    /**
     * @param $input_type
     * @param $self
     * @param $screen
     * @return string
     */
    private function _generate_input_field($input_type, $input_data)
    {
        switch($input_type) {

            case "hr":

                ob_start();
                ?>

                <div class="form-group field-event">
                    <hr class="<?= $input_data['block_text']; ?>">
                </div>

                <?php
                return ob_get_clean();
                break;

            case "blocktext":

                $input_data['field_label'] = $this->_format_required_labels($input_data['field_label'], $input_data['required']);

                ob_start();
                ?>

                <div class="form-group field-event">
                    <label class="col-lg-2 control-label"><?= $input_data['field_label']; ?></label>
                    <div class="col-lg-10">
                        <?php
                        echo $option_labels_a = $input_data['block_text'];
                        ?>
                    </div>
                </div>

                <?php
                return ob_get_clean();
                break;

            case "checkboxes":

                $horizontal = $input_data['horizontal'] == 1 ? 'checkbox-horizontal' : '';
                $input_data['field_label'] = $this->_format_required_labels($input_data['field_label'], $input_data['required']);

                ob_start();
                ?>

                <div class="form-group field-event">
                    <label class="col-lg-2 control-label" for="clients-<?= $input_data['model_field']; ?>"><?= $input_data['field_label']; ?></label>
                    <div class="col-lg-10">
                        <input name="<?= $input_data['model_label']; ?>[<?= $input_data['model_field']; ?>]" type="hidden">
                        <div id="clients-form" class="selectors-shift-left">

                            <?php
                            $option_labels_a = $input_data['option_labels_a'];
                            $option_values_a = $input_data['option_values_a'];
                            $option_values_selected_a = $input_data['value'] != null ? json_decode($input_data['value'],true) : [];

                            $c = 0;
                            foreach($option_values_a as $key => $val) {
                                $c++;
                                $required = $c == 1 && $input_data['required'] == 1 ? 'data-parsley-mincheck="1" data-parsley-errors-container="#error-box-'.$input_data['model_field'].'" required' : '';
                                ?>

                                <div class="col-lg-8 checkbox <?= $horizontal ?>">

                                <?php if(in_array($key, $option_values_selected_a)): ?>
                                    <input <?= $required; ?> id="<?= $input_data['model_field']; ?>_<?= $key ?>" class="client_type" type="checkbox" checked="checked" name="<?= $input_data['model_label']; ?>[<?= $input_data['model_field']; ?>][]" value="<?= $key; ?>">
                                <?php else: ?>
                                    <input <?= $required; ?> id="<?= $input_data['model_field']; ?>_<?= $key ?>" class="client_type" type="checkbox" name="<?= $input_data['model_label']; ?>[<?= $input_data['model_field']; ?>][]" value="<?= $key; ?>">
                                <?php endif; ?>

                                <label for="<?= $input_data['model_field']; ?>_<?= $key ?>"><?= $val ?></label>

                                <?php if(!empty($option_labels_a[$key])): ?>
                                    <div class='well well-infobox client_desc'><?= $option_labels_a[$key] ?></div>
                                <?php endif; ?>
                                </div>

                            <?php
                            }
                            ?>

                        </div>
                    </div>
                </div>
                <div class="col-lg-offset-2" id="error-box-<?= $input_data['model_field']; ?>"></div>
                <div class="col-lg-offset-2">
                    <p class="help-block"><?= $input_data['help_text']; ?></p>
                </div>

                <?php
                return ob_get_clean();
                break;

            case "radio":

                $required = $input_data['required'] == 1 ? 'required' : '';
                $horizontal = $input_data['horizontal'] == 1 ? 'radio-horizontal' : '';
                $input_data['field_label'] = $this->_format_required_labels($input_data['field_label'], $input_data['required']);

                ob_start();
                ?>
                <div class="form-group field-clients-<?= $input_data['model_field']; ?>">
                    <label class="col-lg-2 control-label" for="clients-<?= $input_data['model_field']; ?>"><?= $input_data['field_label']; ?></label>
                    <div class="col-lg-8">
                        <div class="selectors-shift-left">

                            <?php
                            $option_values_a = $input_data['option_values_a'];
                            $option_value_selected_a = $input_data['value'];

                            $c = 0;
                            foreach($option_values_a as $key => $val) {
                                $c++;
                                $required = $c == 1 && $input_data['required'] == 1 ? 'data-parsley-errors-container="#error-box-'.$input_data['model_field'].'" required' : '';

                            ?>
                                <div class="col-lg-8 radio <?= $horizontal ?>">
                                    <input type="radio" id="<?= $input_data['model_label']; ?>[<?= $input_data['model_field']; ?>]-<?php echo $key?>" name="<?= $input_data['model_label']; ?>[<?= $input_data['model_field']; ?>]" value="<?= $key; ?>" <?php checked($key, $option_value_selected_a); ?> <?= $required; ?>>
                                    <label for="<?= $input_data['model_label']; ?>[<?= $input_data['model_field']; ?>]-<?php echo $key?>" style="">
                                        <?php echo $val; ?>
                                    </label>
                                </div>

                            <?php
                            }
                            ?>
                            <div class="col-lg-offset-2" id="error-box-<?= $input_data['model_field']; ?>"></div>
                            <div class="col-lg-offset-2">
                                <p class="help-block"><?= $input_data['help_text']; ?></p>
                            </div>

                        </div>
                    </div>
                </div>
                <?php
                return ob_get_clean();
                break;

            case "dropdown":

                $required = $input_data['required'] == 1 ? 'required' : '';
                $input_data['field_label'] = $this->_format_required_labels($input_data['field_label'], $input_data['required']);
                ob_start();
                ?>
                <div class="form-group field-clients-<?= $input_data['model_field']; ?>">
                    <label class="col-lg-2 control-label" for="clients-<?= $input_data['model_field']; ?>"><?= $input_data['field_label']; ?></label>

                    <div class="col-lg-8">
                        <div class="">
                            <?php
                            $option_values_a = $input_data['option_values_a'];
                            $option_value_selected_a = $input_data['value'];
                            ?>
                            <div class="col-lg-8">
                            <select id="<?= $input_data['model_label']; ?>[<?= $input_data['model_field']; ?>]" style="width:auto;" class="form-control" name="<?= $input_data['model_label']; ?>[<?= $input_data['model_field']; ?>]" <?= $required; ?>>
                                <option value="">Select...</option>
                                    <?php

                                foreach($option_values_a as $key => $val) {
                                    ?>
                                    <?php if($key == $option_value_selected_a): ?>
                                        <option value="<?= $key; ?>" selected><?= $val; ?></option>
                                    <?php else: ?>
                                        <option value="<?= $key; ?>"><?= $val; ?></option>
                                    <?php endif; ?>
                                    <?php
                                }
                                    ?>
                            </select>
                            </div>
                            <div class="col-lg-offset-2">
                                <p class="help-block"><?= $input_data['help_text']; ?></p>
                            </div>
                        </div>
                    </div>
                </div>
                <?php
                return ob_get_clean();
                break;

            case "applic_keywords":

                $required = $input_data['required'] == 1 ? 'data-parsley-errors-container="#error-box-'.$input_data['model_field'].'" required' : '';

                $input_data['field_label'] = $this->_format_required_labels($input_data['field_label'], $input_data['required']);
                ob_start();
                ?>
                <script type="text/javascript">
                    jQuery(document).ready(function () {
                        // initialize jQuery stuff after page load
                        jQuery(function(){
                            jQuery('#clients-applic_keywords').select2({
                                createTag: function(params) {
                                    var open_tags = <?= $input_data['custom_keywords_allowed'] ?>;
                                    var term = jQuery.trim(params.term);
                                    if(open_tags == 0) {
                                        return undefined;
                                    } else {
                                        return {
                                            id: term,
                                            text: term
                                        }
                                    }
                                },
                                data: [<?= $input_data['keywords_unselected']; ?>],
                                tags: true,
                                tokenSeparators: [','],
                                placeholder: "Add your tags here",
                                selectOnClose: false,
                                allowClear: true
                            });
                        })
                    });
                </script>

                <div class="form-group field-event">
                    <label class="col-lg-2 control-label" for="clients-<?= $input_data['model_field']; ?>"><?= $input_data['field_label']; ?></label>
                    <div class="col-lg-10">
                        <input name="<?= $input_data['model_label']; ?>[<?= $input_data['model_field']; ?>]" type="hidden">
                        <select id="clients-<?= $input_data['model_field']; ?>" class="form-control" name="<?= $input_data['model_label']; ?>[<?= $input_data['model_field']; ?>][]" multiple="multiple" style="width:100%" <?= $required; ?>>
                            <?php
                            if(isset($input_data['keywords_selected'])) {
                                foreach ($input_data['keywords_selected'] AS $key_id => $keyword_label) {
                                    ?>
                                    <option value="<?= $key_id; ?>" selected><?= $keyword_label ?></option>
                                    <?php
                                }
                            }
                            ?>
                        </select>
                    </div>
                </div>
                <div class="col-lg-offset-2" id="error-box-<?= $input_data['model_field']; ?>"></div>
                <div class="col-lg-offset-2">
                    <?php if($input_data['help_text'] !== ''): ?>
                        <p class="help-block"><?= $input_data['help_text']; ?></p>
                    <?php elseif($input_data['custom_keywords_allowed'] == 1): ?>
                        <p class="help-block">Select related keywords from dropdown list or ADD your own. Use ENTER or COMMA to separate keywords.</p>
                    <?php else: ?>
                        <p class="help-block">Select related keywords from dropdown list.</p>
                    <?php endif; ?>
                </div>
                <?php
                return ob_get_clean();
                break;

            case "textfield":

                $max_width = $input_data['max_width'] != null ? 'max-width: '.$input_data['max_width'].'px' : '';
                $required = $input_data['required'] == 1 ? 'required' : '';
                $filter_type = $input_data['filter'] != null && $input_data['filter'] != 'general' ? 'data-parsley-type="'.$input_data['filter'].'"' : '';
                $input_data['field_label'] = $this->_format_required_labels($input_data['field_label'], $input_data['required']);

                ob_start();
                ?>

                <div class="form-group field-clients-name">
                    <label class="col-lg-2 control-label" for="clients-<?= $input_data['model_field']; ?>"><?= $input_data['field_label']; ?></label>
                    <div class="col-lg-8">
                        <input type="text" id="clients-<?= $input_data['model_field']; ?>" class="form-control" name="<?= $input_data['model_label']; ?>[<?= $input_data['model_field']; ?>]" maxlength="255" value="<?= $input_data['value']; ?>" style="<?= $max_width; ?>" <?= $filter_type; ?> <?= $required; ?>>
                    </div>
                </div>
                <div class="col-lg-offset-2">
                    <p class="help-block"><?= $input_data['help_text']; ?></p>
                </div>

                <?php
                return ob_get_clean();
                break;

            case "textarea":

                $required = $input_data['required'] == 1 ? 'required' : '';
                $input_data['field_label'] = $this->_format_required_labels($input_data['field_label'], $input_data['required']);
                ob_start();
                ?>

                <div class="form-group field-clients-<?= $input_data['model_field']; ?>">
                    <label class="col-lg-2 control-label" for="clients-<?= $input_data['model_field']; ?>"><?= $input_data['field_label']; ?></label>
                    <div class="col-lg-8">
                        <textarea name="<?= $input_data['model_label']; ?>[<?= $input_data['model_field']; ?>]" id="clients-<?= $input_data['model_field']; ?>" class="form-control" <?= $required; ?>><?= $input_data['value']; ?></textarea>
                    </div>
                </div>
                <div class="col-lg-offset-2">
                    <p class="help-block"><?= $input_data['help_text']; ?></p>
                </div>

                <?php
                return ob_get_clean();
                break;

            case "richtextarea":

                $required = $input_data['required'] == 1 ? 'required' : '';
                $input_data['field_label'] = $this->_format_required_labels($input_data['field_label'], $input_data['required']);
                ob_start();
                ?>

                <div class="form-group field-clients-<?= $input_data['model_field']; ?>">
                    <label class="col-lg-2 control-label" for="clients-<?= $input_data['model_field']; ?>"><?= $input_data['field_label']; ?></label>
                    <div class="col-lg-8">
                        <textarea name="<?= $input_data['model_label']; ?>[<?= $input_data['model_field']; ?>]" id="clients-<?= $input_data['model_field']; ?>" class="rich-textarea form-control" <?= $required; ?>><?= $input_data['value']; ?></textarea>
                    </div>
                </div>
                <div class="col-lg-offset-2">
                    <p class="help-block"><?= $input_data['help_text']; ?></p>
                </div>

                <?php
                return ob_get_clean();
                break;

            case "telephone":

                $max_width = $input_data['max_width'] != null ? $input_data['max_width'].'px' : '250px';
                $required = $input_data['required'] == 1 ? 'required' : '';
                $input_data['field_label'] = $this->_format_required_labels($input_data['field_label'], $input_data['required']);
                ob_start();
                ?>

                <div class="form-group field-clients-<?= $input_data['model_field']; ?>">
                    <label class="col-lg-2 control-label" for="clients-<?= $input_data['model_field']; ?>"><?= $input_data['field_label']; ?></label>
                    <div class="col-lg-8">
                        <input type="tel" id="clients-<?= $input_data['model_field']; ?>" class="telephone form-control" name="<?= $input_data['model_label']; ?>[<?= $input_data['model_field']; ?>]" maxlength="255" value="<?= $input_data['value']; ?>" style="max-width: <?= $max_width; ?>" <?= $required; ?>>
                    </div>
                </div>
                <div class="col-lg-offset-2">
                    <p class="help-block"><?= $input_data['help_text']; ?></p>
                </div>

                <?php
                return ob_get_clean();
                break;

            case "com_prefs":

                $required = $input_data['required'] == 1 ? ' aria-required="true" required' : '';
                $input_data['field_label'] = $this->_format_required_labels($input_data['field_label'], $input_data['required']);
                ob_start();
                ?>
                <div class="form-group field-clients-<?= $input_data['model_field']; ?>_temp">
                    <label class="col-lg-2 control-label" for="clients-<?= $input_data['model_field']; ?>_temp"><?= $input_data['field_label']; ?></label>
                    <div class="col-lg-offset-2">
                        <p class="help-block"><?= $input_data['help_text']; ?></p>
                    </div>
                    <div class="col-lg-8">
                        <div class="selectors-shift-left">
                            <?php echo $this->_generate_communication_prefs($input_data['value'],$input_data['horizontal']); ?>
                        </div>
                    </div>
                </div>
                <?php
                return ob_get_clean();
                break;

            case "client_types_c":

                $required = $input_data['required'] == 1 ? 'required' : '';
                $input_data['field_label'] = $this->_format_required_labels($input_data['field_label'], $input_data['required']);
                ob_start();
                ?>

                <div class="form-group field-event">
                    <label class="col-lg-2 control-label" for="clients-<?= $input_data['model_field']; ?>"><?= $input_data['field_label']; ?></label>
                    <div class="col-lg-10">
                        <input name="<?= $input_data['model_label']; ?>[<?= $input_data['model_field']; ?>]" type="hidden">
                        <div id="clients-form" class="selectors-shift-left">
                             <?= $this->_generate_checkboxes_client_types($input_data,'checkbox'); ?>
                        </div>
                    </div>
                </div>
                <div class="col-lg-offset-2" id="error-box-<?= $input_data['model_field']; ?>"></div>
                <div class="col-lg-offset-2">
                    <p class="help-block"><?= $input_data['help_text']; ?></p>
                </div>

                <?php
                return ob_get_clean();
                break;

            case "client_types_r":

                $required = $input_data['required'] == 1 ? 'required' : '';
                $input_data['field_label'] = $this->_format_required_labels($input_data['field_label'], $input_data['required']);
                ob_start();
                ?>

                <div class="form-group field-event">
                    <label class="col-lg-2 control-label" for="clients-<?= $input_data['model_field']; ?>"><?= $input_data['field_label']; ?></label>
                    <div class="col-lg-10">
                        <input name="<?= $input_data['model_label']; ?>[<?= $input_data['model_field']; ?>]" type="hidden">
                        <div id="clients-form" class="selectors-shift-left">
                            <?= $this->_generate_checkboxes_client_types($input_data,'radio'); ?>
                        </div>
                    </div>
                </div>
                <div class="col-lg-offset-2" id="error-box-<?= $input_data['model_field']; ?>"></div>
                <div class="col-lg-offset-2">
                    <p class="help-block"><?= $input_data['help_text']; ?></p>
                </div>

                <?php
                return ob_get_clean();
                break;

            default:
                break;
        }
    }

    /**
     * @param string $label
     * @param int $required
     * @return string
     */
    private function _format_required_labels($label = '', $required = '')
    {
        $label = $required == 1 ? $label.'*' : $label;

        return $label;
    }

    /**
     * @param int $temp_prefs
     * @return mixed
     */
    private function _get_preference_tag($temp_prefs = 0)
    {
        $prefs = ['None', 'e-Mail', 'Cell Text Msg', 'Both (recommended)'];

        return $prefs[$temp_prefs];
    }

    /**
     * @param string $links
     * @return string
     */
    private function _generate_social_links($links = "")
    {
        $links = explode("\r\n", $links);

        ob_start();
        foreach($links as $key => $link) :
            if($link != ''):
         ?>
                <a href="<?php echo $link?>" target="_blank" title="<?php echo $link?>"><?php echo $link; ?></a>
                <br>
         <?php
            endif;
        endforeach;

        return ob_get_clean();
    }

    /**
     * @param array $mercs
     * @return string
     */
    private function _generate_merchandise_list($mercs = [])
    {
        ob_start();

        if(count($mercs)):
            foreach($mercs as $key => $merc):
         ?>
                <span class="tooltip-info label label-info" data-original-title="" style="white-space: nowrap;  text-decoration: none; cursor: default;"><?php echo $merc; ?></span>
        <?php
            endforeach;
        endif;

        return ob_get_clean();
    }

    /**
     * @param array $tags
     * @return string
     */
    private function _generate_tag_list($tags = [])
    {
        ob_start();

        if(count($tags)):
            foreach($tags as $key => $tag):
                ?>
                <span class="tooltip-info label label-info" data-original-title="" style="white-space: nowrap;  text-decoration: none; cursor: default;"><?php echo $tag; ?></span>
                <?php
            endforeach;
        endif;

        return ob_get_clean();
    }

    /**
     * @param int $temp_prefs
     * @return string
     */
    private function _generate_communication_prefs($temp_prefs = 3,$horizontal = 0)
    {
        $prefs = ['None', 'e-Mail', 'Cell Text Msg', 'Both (recommended)'];
        $horizontal = $horizontal == 1 ? 'radio-horizontal' : '';
        $temp_prefs = $temp_prefs == null ? 3 : $temp_prefs;

        ob_start();
        foreach($prefs as $key => $val) {
            // $checked = $key == $temp_prefs ? 'checked' : '';
            ?>
                <div class="col-lg-8 radio <?= $horizontal; ?>">

                    <input type="radio" id="Clients[com_prefs]-<?php echo $key?>" name="Clients[com_prefs]" value="<?php echo $key?>" <?php checked($key,$temp_prefs); ?>>
                    <label for="Clients[com_prefs]-<?php echo $key?>" style="white-space: nowrap">
                       <?php echo $val?>
                    </label>
                </div>
            <?php
        }

        ?>
        <div class="form-group field-client-com_prefs_temp">
            <div class="col-lg-8">
                <input type="hidden" name="Clients[com_prefs_temp]">
                <div id="client-com_prefs_temp"></div>
            </div>
        </div>
        <?php

        return ob_get_clean();
    }

    /**
     * @param $fields
     * @param $attributes
     * @return string
     */
    private function _generate_payment_screen_field($fields, $attributes)
    {
        $attributes = json_decode($attributes, true);

        ob_start();
        ?>
        <div id="letter" class="content-inner">
            <div id="curriculum-vitae" class="block">

                    <div id="block-content">
                        <div class="clients-form">
                            <?php echo $fields; ?>
                        </div>
                    </div>
                <div class="clear"></div>
            </div>
        </div>
        <?php

        return ob_get_clean();
    }

    /**
     * @param $fields
     * @param $attributes
     * @return string
     */
    private function _generate_terms_screen_field($fields, $attributes)
    {
        $attributes = json_decode($attributes, true);

        ob_start();
        ?>
        <div id="letter" class="content-inner">
            <div id="curriculum-vitae" class="block">

                    <div id="block-content">
                        <div class="clients-form">
                            <?php echo $fields; ?>
                        </div>
                    </div>
                <div class="clear"></div>
            </div>
        </div>
        <?php

        return ob_get_clean();
    }

    /**
     * @param $fields
     * @param $attributes
     * @return string
     */
    private function _generate_amenities_screen_field($fields, $attributes)
    {
        $attributes = json_decode($attributes, true);

        ob_start();
        ?>
        <div id="letter" class="content-inner">
            <div id="curriculum-vitae" class="block">

                    <div class="block-content">
                        <div class="clients-form">
                            <form data-parsley-validate="" class="form-horizontal" name="step-3" action="<?php echo admin_url('admin-ajax.php'); ?>" id="clients-form" method="post" enctype="multipart/form-data">
                                <input type="hidden" name="_csrf" value="">
                                <input type="hidden" name="dwc_screen_name" value="amenities">
                                <?php wp_nonce_field('dwc_form_submission', 'dwc_verify_submission'); ?>
                                <input type="hidden" name="action" value="dwc_form_submission">
                                <?php echo $fields; ?>
                            </form>
                        </div>
                    </div>
                <div class="clear"></div>
            </div>
        </div>
        <?php

        return ob_get_clean();
    }

    /**
     * @param $fields
     * @param $attributes
     * @return string
     */
    private function _generate_uploads_screen_field($fields, $attributes)
    {
        $attributes = json_decode($attributes, true);

        ob_start();
        ?>
        <div id="letters" class="content-inner">
            <div id="curriculum-vitae" class="block">

                    <div class="block-content">
                        <div class="clients-form">
                            <form class="form-horizontal" name="step-2" action="<?php echo admin_url('admin-ajax.php'); ?>" id="clients-form" method="post" enctype="multipart/form-data">
                                <input type="hidden" name="_csrf" value="">
                                <input type="hidden" name="dwc_screen_name" value="uploads">
                                <?php wp_nonce_field('dwc_form_submission', 'dwc_verify_submission'); ?>
                                <input type="hidden" name="action" value="dwc_form_submission">
                                <?php echo $fields; ?>
                            </form>
                        </div>
                    </div>
                <div class="clear"></div>
            </div>
        </div>
        <?php

        return ob_get_clean();
    }

    /**
     * @return string
     */
    private function _get_base_url()
    {
        global $wp;
        $base_url = home_url(add_query_arg(array(), $wp->request));
        return $base_url . "/";
    }

    /**
     * @param $fields
     * @param $client_a
     * @return string
     */
    private function _generate_profile_screen_field($fields, $client_a, $event_a, $pre_launch_count = 0)
    {
        global $wp;
        $current_url = home_url(add_query_arg([], $wp->request));

        ob_start();

        ?>
        <div id="" class="content-inner">
            <div id="" class="block">
                <?php if($event_a['event_status'] == 0): ?>
                <div id="">
                    <h3><span style="color:#ff6b6b">Pre-Launch Applicants:</span> <?= $pre_launch_count; ?> of 5</h3>
                </div>
                <?php endif; ?>
                    <div id="">
                        <?php if(isset($_GET['rfc']) && $_GET['rfc'] != ''): ?>
                            <div class="well well-sm" style="margin-bottom:10px; margin-top:10px;">
                                <h4>This page has been pre-filled for your convenience.
                                Please review the details for accuracy and then click <strong>Save & Continue</strong> at the bottom
                                    of this page.</h4>
                            </div>

                        <?php elseif(isset($_GET['return'])): ?>

                            <div class="well well-sm" style="margin-bottom:20px; margin-top:10px;">
                                <i style="float:left; margin-right:10px;" class="fa fa-2x fa-exclamation-circle blue" aria-hidden="true"></i>
                                <h4>If you ALREADY filled out an application for this event and need to make changes
                                    or payments, you must use the original application profile link.</h4>
                            </div>

                            <div class="" style="margin-top: 0px; margin-right: auto; margin-bottom: 20px; margin-left: auto; width: 96%; align-content: center;">
                                <div class="" style="margin-top: 10px; margin-bottom: 10px;">
                                    <h4>To retrieve your application profile link, you can do one of the following:</h4>
                                </div>

                                <ol class="return_vendor_list">
                                    <li>Find the original email we sent to you that contains your application profile link.</li>
                                    <li><a href="https://app.registerone.com/signup">Create a free RegisterONE account</a> to
                                        manage all your applications in one place.</li>
                                    <li>Enter your original registration email below and we will re-send the link to you.</li>
                                </ol>
                            </div>

                        <?php elseif(!isset($client_a['name'])): ?>
                            <!-- New Application -->

                        <?php endif; ?>
                    </div>
                    <div class="block-content">
                        <div class="clients-form">
                            <form data-parsley-validate="" id="clients-form" class="form-horizontal" name="step-1" action="<?php echo admin_url('admin-ajax.php'); ?>" method="post" enctype="multipart/form-data">
                                <input type="hidden" name="_csrf" value="">
                                <input type="hidden" name="dwc_screen_name" value="profile">
                                <?php wp_nonce_field('dwc_form_submission', 'dwc_verify_submission'); ?>
                                <input type="hidden" name="action" value="dwc_form_submission">
                                <?php echo $fields; ?>
                            </form>
                        </div>
                    </div>
                <div class="clear"></div>
            </div>
        </div>
        <?php

        return ob_get_clean();
    }

    /**
     * @param $current
     * @param $self
     * @param $screen
     * @return string
     */
    private function _check_step_completeness($current, $self, $screen)
    {
        global $wp_query;

        // Dynamic form_key with url slug
        $attributes['id_hash'] = $wp_query->query_vars['id_hash'] ? $wp_query->query_vars['id_hash'] : 0;
        $request_url = $this->base_request_url . $attributes['id_hash'] . "/";

        switch($screen) {

            case "profile":
                if($current < $self) return "";
                if($current == $self) return "active";
                $errors = 0;
                $attributes = json_decode( $this->event_attributes, true);

                if(isset($attributes['applicant_a']['client_type'])) {
                    $client_types = json_decode($attributes['applicant_a']['client_type'], true);
                    if(!count($client_types)) {
                        $errors++;
                    }
                }else{
                    $errors++;
                }

                if(!isset($attributes['client_a']['name']) || $attributes['client_a']['name'] == '')
                    $errors++;
                if(!isset($attributes['client_a']['first_name']) || $attributes['client_a']['first_name'] == '')
                    $errors++;
                if(!isset($attributes['client_a']['last_name']) || $attributes['client_a']['last_name'] == '')
                    $errors++;
                if(!isset($attributes['client_a']['email']) || $attributes['client_a']['email'] == '')
                    $errors++;
                if(!isset($attributes['client_a']['phone']) || $attributes['client_a']['phone'] == '')
                    $errors++;
                if(!isset($attributes['client_a']['com_prefs']) || $attributes['client_a']['com_prefs'] == '')
                    $errors++;

                if($errors) {
                    return "error";
                }else {
                    return "complete";
                }
                break;

            case "uploads":
                if($current < $self) return "";
                if($current == $self) return "active";
                if($current > $self) {

                    $attributes = json_decode($this->event_attributes, true);

                    if(isset($_GET['pk']) && $_GET['pk'] != '') {
                        /*
                        'uploads'
                        'terms'
                        'map'
                        'fee'
                         * */
                        if(in_array('uploads', $attributes['error_details_a'])){
                            return "error";
                        }else{
                            return "complete";
                        }

                     }else{
                        return "error";
                    }
                }
                break;

            case "amenities":
                if($current < $self) return "";
                if($current == $self) return "active";
                if($current > $self) {

                    $attributes = json_decode($this->event_attributes, true);

                    if(isset($_GET['pk']) && $_GET['pk'] != '') {
                        /*
                        'uploads'
                        'terms'
                        'map'
                        'fee'
                         * */
                        if(in_array('map', $attributes['error_details_a'])){
                            return "error";
                        }else{
                            return "complete";
                        }

                    }else{
                        return "complete";
                    }
                }
                break;

            case "terms":
                if($current < $self) return "";
                if($current == $self) return "active";
                if($current > $self) {
                    $attributes = json_decode($this->event_attributes, true);

                    if(isset($_GET['pk']) && $_GET['pk'] != '') {
                        /*
                        'uploads'
                        'terms'
                        'map'
                        'fee'
                         * */
                        if(in_array('terms', $attributes['error_details_a'])){
                            return "error";
                        }else{
                            return "complete";
                        }

                    }else{
                        return "complete";
                    }
                }
                break;

            case "payment":
                if($current < $self) return "";
                // if($current == $self) return "active";
                if($current == $self) {
                    $attributes = json_decode($this->event_attributes, true);

                    if(isset($_GET['pk']) && $_GET['pk'] != '') {
                        /*
                        'uploads'
                        'terms'
                        'map'
                        'fee'
                         * */
                        if(in_array('payment', $attributes['error_details_a'])){
                            return "error";
                        }else{
                            return "complete";
                        }

                    }else{
                        return "complete";
                    }
                }
                break;

            default:
                break;
        }
    }

    /**
     * @param $screen
     * @return string
     */
    private function _get_steps_list($screen, $formData)
    {
        global $wp;
        $current_url = home_url(add_query_arg([], $wp->request));

        $profile_status = 0;
        
        if(isset($_GET['pk']) && $_GET['pk'] != '') {
            $profile_status = 1;
        }

        if(isset($formData['nav_steps_list'])){
            $steps = $formData['nav_steps_list'][0];
            $nav_label_replace_a = $formData['nav_steps_list'][1];
        }else{
            $steps = ['profile', 'uploads', 'amenities', 'terms', 'payment'];
        }

        $current_step = array_search($screen, $steps);

        ob_start();

        foreach($steps as $key => $val) {
            $is_last_step = (count($steps) == $current_step+1 && $current_step == $key)  ? true : false;
            $mt = str_replace('.', '', microtime(true));
            $nav_label = isset($nav_label_replace_a[$val]) ? $nav_label_replace_a[$val] : $val;
            $li_class = $this->_check_step_completeness($current_step+1, $key+1, $steps[$key]);
            $step_style = ($li_class == 'active' || $is_last_step) ? 'font-weight:bold;' : '';
            ?>
            <li id="<?php echo $key; ?>" data-step="<?php echo ($key + 1); ?>" class="<?= $li_class; ?>">
                <span class="step"><?php echo ($key + 1) ?></span>
                <span class="title" style="<?= $step_style; ?>">
                    <?php if($profile_status && isset($_GET['rfc'])): ?>
                        <a href="<?php echo $current_url . "/?profile&pk=" . $_GET['pk'] . "&rfc=" . $_GET['rfc']; ?>"><?= ucfirst($nav_label); ?></a>
                    <?php elseif($profile_status): ?>
                        <a href="<?php echo $current_url . "/?" . $val  . "&pk=" . $_GET['pk'] . "&v=" . $mt; ?>"><?= ucfirst($nav_label); ?></a>
                    <?php else: ?>
                        <a href="<?php echo $current_url; ?>/"><?php echo ($val == "terms") ? ucfirst($nav_label) : ucfirst($nav_label); ?></a>
                    <?php endif; ?>
                </span>
            </li>
            <?php
        }

        return ob_get_clean();
    }

    /**
     * @param string $screen
     * @param string $form
     * @return string
     */
    private function _set_application_steps($screen = "", $form = "", $formData = "")
    {

        $form .= "<ul id=\"steps\" class=\"steps\">";

        $form .= $this->_get_steps_list($screen,$formData);

        $form .= "</ul>";

        return $form;
    }

    /**
     * @param $event_a
     * @param string $form
     * @return string
     */
    private function _set_application_title($event_a, $form = "")
    {
        global $wp;
        $current_url = home_url(add_query_arg([], $wp->request));

        $event_a = json_decode($event_a, true);
        $attributes = json_decode( $this->event_attributes, true);

        if(isset($event_a['event_a']['home_url'])){
            $home_url = "<a href='{$event_a['event_a']['home_url']}'>Home</a>";
        }else{
            $home_url = "<a href='/'>Home</a>";
        }

        $status = $event_a['application_status'] == 1 ? " <span class='green-alert'>Complete!</span>" : " <span class='red-alert'>Incomplete</span>";

        if(isset($attributes['client_a']['name']) && $attributes['client_a']['name'] != ''):
            $base_url = $this->_get_base_url();
            $login_label = "{$attributes['client_a']['name']} [<a href='{$base_url}'>not you?</a>]";

            $pk = $attributes['applicant_a']['profile_key'];

            $pdf_link = "<a title='Download Profile' class='tooltip' href='{$this->base_site_url}/pdf/download?pk={$pk}'>
            <i class='fa fa-lg fa-file-pdf-o' aria-hidden='true'></i>
            </a>";
        else:
            $login_label = "<div style='float:right; text-align: left;'>
                <h5><span style='font-style:italic' class='green-alert'>Need to edit your application?</span> [<a href='{$current_url}/?return=1'>click here</a>]</h5>
            </div>";
            $pdf_link = '';
        endif;

        if(isset($event_a['event_a']['title'])) {
            $form_header = "<div class='pull-left form-title-header'><h2>{$home_url} > {$event_a['event_a']['title']}</h2></div>";

            $form_header .= "<div class='pull-right' style='margin-bottom: 6px;'>
                <div class='pull-right'>
                    <h5>{$login_label}</h5>
                </div>
                <div class='pull-right' style='clear:both'>
                    <h4>Status: {$status} {$pdf_link}</h4>
                </div>
            </div>";

            return $form . $form_header;
        }else{
            return "";
        }
    }

    /**
     * @param $event_a
     * @param string $form
     * @return string
     */
    private function _get_signature_status($attributes, $request_url)
    {
        global $wp_query;

        // Dynamic form_key with url slug
        $attributes['id_hash'] = $wp_query->query_vars['id_hash'] ? $wp_query->query_vars['id_hash'] : 0;
        $request_url = $this->base_request_url . $attributes['id_hash'] . "/";

        $no_signatures_required = $terms_require_sign = $contract_require_sign = $no_contract_signature_required = false;
        $terms_signed = $contract_signed = $approval_status = '';

        if(isset($_GET['pk']) && $_GET['pk'] != '') {

            $approval_status = $attributes['applicant_a']['approval_status'];
            $terms_id = $attributes['event_a']['terms_id'];
            $contract = $attributes['event_a']['contract'];
            $c_cfg2 = $attributes['event_a']['c_cfg2'];

            if ($approval_status != 1) {
                if ($terms_id > 0) {
                    $terms_require_sign = true;
                } elseif ($contract == 1 && $c_cfg2 == 1) {
                    $contract_require_sign = true;
                }

            } elseif ($approval_status == 1) {
                if ($terms_id > 0) {
                    $terms_require_sign = true;
                }
                if ($contract == 1 && $c_cfg2 == 2) {
                    $contract_require_sign = true;
                }
            }

            if($attributes['event_a']['terms_id'] == null && $attributes['event_a']['contract'] == 0){
                $no_signatures_required = true;
            }else{
                $applicant_signature = json_decode($this->_get_data("signature", $attributes), true);
            }

            if(isset($applicant_signature['signature'])) {
                foreach($applicant_signature['signature'] AS $signature){
                    if($signature['terms_scope'] == 1){
                        $terms_signed = true;
                    }
                    if($signature['terms_scope'] == 2){
                        $contract_signed = true;
                    }
                }
            }
        }

        $sign_status_passed = true;

        // no terms or contract
        if($no_signatures_required){
            return true;

        }else{
            if($terms_require_sign && !$terms_signed) {
                $sign_status_passed = false;
            }
            if($contract_require_sign && !$contract_signed) {
                $sign_status_passed = false;
            }
        }
        return $sign_status_passed;
    }

    /*
    * @param
    * @return array('mem_limit','upload_max_filesize','post_max_size')
    */
    private function _get_php_limits()
    {
        $php_env_limits['mem_limit'] = ini_get('memory_limit');
        $php_env_limits['upload_max_filesize'] = ini_get('upload_max_filesize');
        $php_env_limits['post_max_size'] = ini_get('post_max_size');

        // Memory Limit: $this->_get_php_limits()['mem_limit']
        // Max Upload Size: $this->_get_php_limits()['upload_max_filesize']
        // Max Post Size: $this->_get_php_limits()['post_max_size']

        return $php_env_limits;
    }

    /**
     * @param $form
     * @return string
     */
    private function _wrap_with_parents($form)
    {
        ob_start();

        ?>
        <div id="wrapper" class="wrapper">
            <div id="container">
                <div id="apply">
                    <div class="content">
                        <?php echo $form; ?>
                    </div>
                    <div class="clear"></div>
                </div>
                <?php if( isset($_GET['pk']) && $_GET['pk'] != null ): ?>
                    <div class="bottom-right-container">
                       BOOKMARK this page so you can edit your application later.
                    </div>
                <?php endif; ?>
            </div>
            <div id="poweredby">
            <?php
            if ($this->footer_powered_by_option == 1){
                ?>
                Powered by <a href="https://www.registerone.com">RegisterONE</a>
                <?php
            }
            ?>
            v1.1.7.2
            </div>
        </div>
        <?php

        return ob_get_clean();
    }
}